from ..exceptions import WQXException
from .ReferenceMethod import ReferenceMethod
from .SamplePreparation import SamplePreparation
from .SimpleContent import (
  HydrologicCondition,
  HydrologicEvent,
  SampleCollectionEquipmentCommentText,
  SampleCollectionEquipmentName
)
from yattag import Doc

class SampleDescription:
  """Basic identification information for the sample collected as part of a monitoring activity."""

  __sampleCollectionMethod: ReferenceMethod
  __sampleCollectionEquipmentName: SampleCollectionEquipmentName
  __sampleCollectionEquipmentCommentText: SampleCollectionEquipmentCommentText
  __samplePreparation: SamplePreparation
  __hydrologicCondition: HydrologicCondition
  __hydrologicEvent: HydrologicEvent

  def __init__(self, o=None, *,
    sampleCollectionMethod:ReferenceMethod = None,
    sampleCollectionEquipmentName:SampleCollectionEquipmentName = None,
    sampleCollectionEquipmentCommentText:SampleCollectionEquipmentCommentText = None,
    samplePreparation:SamplePreparation = None,
    hydrologicCondition:HydrologicCondition = None,
    hydrologicEvent:HydrologicEvent = None
  ):
    if isinstance(o, SampleDescription):
      # Assign attributes from object without typechecking
      self.__sampleCollectionMethod = o.sampleCollectionMethod
      self.__sampleCollectionEquipmentName = o.sampleCollectionEquipmentName
      self.__sampleCollectionEquipmentCommentText = o.sampleCollectionEquipmentCommentText
      self.__samplePreparation = o.samplePreparation
      self.__hydrologicCondition = o.hydrologicCondition
      self.__hydrologicEvent = o.hydrologicEvent
    elif isinstance(o, dict):
      # Assign attributes from dictionary with typechecking
      self.sampleCollectionMethod = o.get('sampleCollectionMethod', default = None)
      self.sampleCollectionEquipmentName = o.get('sampleCollectionEquipmentName', default = None)
      self.sampleCollectionEquipmentCommentText = o.get('sampleCollectionEquipmentCommentText', default = None)
      self.samplePreparation = o.get('samplePreparation', default = None)
      self.hydrologicCondition = o.get('hydrologicCondition', default = None)
      self.hydrologicEvent = o.get('hydrologicEvent', default = None)
    else:
      # Assign attributes from named keywords with typechecking
      self.sampleCollectionMethod = sampleCollectionMethod
      self.sampleCollectionEquipmentName = sampleCollectionEquipmentName
      self.sampleCollectionEquipmentCommentText = sampleCollectionEquipmentCommentText
      self.samplePreparation = samplePreparation
      self.hydrologicCondition = hydrologicCondition
      self.hydrologicEvent = hydrologicEvent

  @property
  def sampleCollectionMethod(self) -> ReferenceMethod:
    """Identifies sample collection or measurement method procedures. Where a documented sample collection method has been employed, this enables the data provider to indicate the documented method that was employed during the field sample collection. Otherwise, the sample collection procedure will best be described in a freeform text."""
    return self.__sampleCollectionMethod
  @sampleCollectionMethod.setter
  def sampleCollectionMethod(self, val:ReferenceMethod) -> None:
    """Identifies sample collection or measurement method procedures. Where a documented sample collection method has been employed, this enables the data provider to indicate the documented method that was employed during the field sample collection. Otherwise, the sample collection procedure will best be described in a freeform text."""
    self.__sampleCollectionMethod = None if val is None else ReferenceMethod(val)

  @property
  def sampleCollectionEquipmentName(self) -> SampleCollectionEquipmentName:
    return self.__sampleCollectionEquipmentName
  @sampleCollectionEquipmentName.setter
  def sampleCollectionEquipmentName(self, val:SampleCollectionEquipmentName) -> None:
    self.__sampleCollectionEquipmentName = SampleCollectionEquipmentName(val)

  @property
  def sampleCollectionEquipmentCommentText(self) -> SampleCollectionEquipmentCommentText:
    return self.__sampleCollectionEquipmentCommentText
  @sampleCollectionEquipmentCommentText.setter
  def sampleCollectionEquipmentCommentText(self, val:SampleCollectionEquipmentCommentText) -> None:
    self.__sampleCollectionEquipmentCommentText = None if val is None else SampleCollectionEquipmentCommentText(val)

  @property
  def samplePreparation(self) -> SamplePreparation:
    return self.__samplePreparation
  @samplePreparation.setter
  def samplePreparation(self, val:SamplePreparation) -> None:
    self.__samplePreparation = None if val is None else SamplePreparation(val)

  @property
  def hydrologicCondition(self) -> HydrologicCondition:
    return self.__hydrologicCondition
  @hydrologicCondition.setter
  def hydrologicCondition(self, val:HydrologicCondition) -> None:
    self.__hydrologicCondition = None if val is None else HydrologicCondition(val)

  @property
  def hydrologicEvent(self) -> HydrologicEvent:
    return self.__hydrologicEvent
  @hydrologicEvent.setter
  def hydrologicEvent(self, val:HydrologicEvent) -> None:
    self.__hydrologicEvent = None if val is None else HydrologicEvent(val)

  def generateXML(self, name:str = 'SampleDescription') -> str:
    doc = Doc()
    asis = doc.asis
    line = doc.line
    tag = doc.tag

    with tag(name):
      if self.__sampleCollectionMethod is not None:
        asis(self.__sampleCollectionMethod.generateXML('SampleCollectionMethod'))
      if self.__sampleCollectionEquipmentName is None:
        raise WQXException("Attribute 'sampleCollectionEquipmentName' is required.")
      line('SampleCollectionEquipmentName', self.__sampleCollectionEquipmentName)
      if self.__sampleCollectionEquipmentCommentText is not None:
        line('SampleCollectionEquipmentCommentText', self.__sampleCollectionEquipmentCommentText)
      if self.__samplePreparation is not None:
        asis(self.__samplePreparation.generateXML('SamplePreparation'))
      if self.__hydrologicCondition is not None:
        line('HydrologicCondition', self.__hydrologicCondition)
      if self.__hydrologicEvent is not None:
        line('HydrologicEvent', self.__hydrologicEvent)

    return doc.getvalue()



from ..exceptions import WQXException
from .SimpleContent import (
  MeasureUnitCode,
  MeasureValue
)
from yattag import Doc

class MeasureCompact:
  """Identifies only the value and the associated units of measure for measuring the observation or analytical result value."""

  __measureValue: MeasureValue
  __measureUnitCode: MeasureUnitCode

  def __init__(self, o=None, *,
    measureValue:MeasureValue = None,
    measureUnitCode:MeasureUnitCode = None
  ):
    if isinstance(o, MeasureCompact):
      # Assign attributes from object without typechecking
      self.__measureValue = o.measureValue
      self.__measureUnitCode = o.measureUnitCode
    elif isinstance(o, dict):
      # Assign attributes from dictionary with typechecking
      self.measureValue = o.get('measureValue', default = None)
      self.measureUnitCode = o.get('measureUnitCode', default = None)
    else:
      # Assign attributes from named keywords with typechecking
      self.measureValue = measureValue
      self.measureUnitCode = measureUnitCode

  @property
  def measureValue(self) -> MeasureValue:
    return self.__measureValue
  @measureValue.setter
  def measureValue(self, val:MeasureValue) -> None:
    self.__measureValue = None if val is None else MeasureValue(val)

  @property
  def measureUnitCode(self) -> MeasureUnitCode:
    return self.__measureUnitCode
  @measureUnitCode.setter
  def measureUnitCode(self, val:MeasureUnitCode) -> None:
    self.__measureUnitCode = None if val is None else MeasureUnitCode(val)

  def generateXML(self, name:str = 'MeasureCompact') -> str:
    doc = Doc()
    line = doc.line
    tag = doc.tag

    with tag(name):
      if self.__measureValue is None:
        raise WQXException("Attribute 'measureValue' is required.")
      line('MeasureValue', self.__measureValue)
      if self.__measureUnitCode is None:
        raise WQXException("Attribute 'measureUnitCode' is required.")
      line('MeasureUnitCode', self.__measureUnitCode)

    return doc.getvalue()



from datetime import datetime
from typing import List, Union
from yattag import Doc
from .exceptions import WQXException

class Header:
  __author: str
  __organization: str
  __title: str
  __creationTime: datetime
  __comment: str
  __dataService: str
  __contactInfo: str
  __notification: List[str]
  __sensitivity: str
  __property: dict

  def __init__(self, o=None, *,
    author = None,
    organization = None,
    title = None,
    creationTime = None,
    comment = None,
    dataService = None,
    contactInfo = None,
    notification = None,
    sensitivity = None,
    property = None
  ):
    if isinstance(o, Header):
      # Assign attributes from object without typechecking
      self.__author = o.author
      self.__organization = o.organization
      self.__title = o.title
      self.__creationTime = o.creationTime
      self.__comment = o.comment
      self.__dataService = o.dataService
      self.__contactInfo = o.contactInfo
      self.__notification = o.notification
      self.__sensitivity = o.sensitivity
      self.__property = o.property
    elif isinstance(o, dict):
      # Assign attributes from dictionary with typechecking
      self.author = o.get('author', default = None)
      self.organization = o.get('organization', default = None)
      self.title = o.get('title', default = None)
      self.creationTime = o.get('creationTime', default = None)
      self.comment = o.get('comment', default = None)
      self.dataService = o.get('dataService', default = None)
      self.contactInfo = o.get('contactInfo', default = None)
      self.notification = o.get('notification', default = None)
      self.sensitivity = o.get('sensitivity', default = None)
      # self.property is tricky because Python confuses it with the decorator
      property = o.get('property', default = None)
      if property is not None and not isinstance(property, dict):
        raise TypeError("Property 'property' must be a dict of 0 or more key/value pairs.")
      self.__property = {} if property is None else property
    else:
      # Assign attributes from named keywords with typechecking
      self.author = author
      self.organization = organization
      self.title = title
      self.creationTime = creationTime
      self.comment = comment
      self.dataService = dataService
      self.contactInfo = contactInfo
      self.notification = notification
      self.sensitivity = sensitivity
      # self.property is tricky because Python confuses it with the decorator
      if property is not None and not isinstance(property, dict):
        raise TypeError("Property 'property' must be a dict of 0 or more key/value pairs.")
      self.__property = {} if property is None else property

  @property
  def author(self) -> str:
    return self.__author
  @author.setter
  def author(self, val:str) -> None:
    self.__author = None if val is None else str(val)

  @property
  def organization(self) -> str:
    return self.__organization
  @organization.setter
  def organization(self, val:str) -> None:
    if not isinstance(val, str):
      raise TypeError("Property 'organization' must be a string.")
    if len(val) < 1:
      raise ValueError("Property 'organization is required.")
    self.__organization = None if val is None else str(val)

  @property
  def title(self) -> str:
    return self.__title
  @title.setter
  def title(self, val:str) -> None:
    if val is not None and not isinstance(val, str):
      raise TypeError("Property 'title' must be a string")
    self.__title = 'WQX' if val is None or len(val) < 1 else val

  @property
  def creationTime(self) -> datetime:
    return self.__creationTime
  @creationTime.setter
  def creationTime(self, val:datetime) -> None:
    if val is not None and not isinstance(val, datetime):
      raise TypeError("Property 'creationTime' must be a datetime.")
    self.__creationTime = datetime.now() if val is None else val

  @property
  def comment(self) -> str:
    return self.__comment
  @comment.setter
  def comment(self, val) -> None:
    if val is not None and not isinstance(val, str):
      raise TypeError("Property 'comment' must be a string, if provided.")
    self.__comment = None if val is None or len(val) < 1 else val

  @property
  def dataService(self) -> str:
    return self.__dataService
  @dataService.setter
  def dataService(self, val) -> None:
    if val is not None and not isinstance(val, str):
      raise TypeError("Property 'dataService' must be a string, if provided.")
    self.__dataService = None if val is None or len(val) < 1 else val

  @property
  def contactInfo(self) -> str:
    return self.__contactInfo
  @contactInfo.setter
  def contactInfo(self, val:str) -> None:
    self.__contactInfo = None if val is None else str(val)

  @property
  def notification(self) -> List[str]:
    return self.__notification
  @notification.setter
  def notification(self, val:Union[str,List[str]]) -> None:
    if val is None:
      self.__notification = []
    elif isinstance(val, list):
      r:List[str] = []
      for x in val:
        if not isinstance(x, str):
          raise WQXException("Attribute 'notification' must be a list of valid URIs or a single URI, if provided.")
        r.append(x)
      self.__notification = r
    else:
      if not isinstance(val, str):
        raise WQXException("Attribute 'notification' must be a list of valid URIs or a single URI, if provided.")
      self.__notification = [val]

  @property
  def sensitivity(self) -> str:
    return self.__sensitivity
  @sensitivity.setter
  def sensitivity(self, val) -> None:
    if val is not None and not isinstance(val, str):
      raise TypeError("Property 'sensitivity' must be a string, if provided.")
    self.__sensitivity = None if val is None or len(val) < 1 else val

  @property
  def property(self) -> dict:
    return self.__property
  @property.setter
  def property(self, val:dict) -> None:
    if val is not None and not isinstance(val, dict):
      raise TypeError("Property 'property' must be a dict of 0 or more key/value pairs.")
    self.__property = {} if val is None else val

  def generateXML(self, name:str = 'Header') -> str:
    doc = Doc()
    tag = doc.tag
    line = doc.line

    with tag(name):
      if self.__author is None:
        raise WQXException("Property 'author' is required.")
      line('Author', self.__author)
      if self.__organization is None:
        raise WQXException("Property 'organization' is required.")
      line('Organization', self.__organization)
      if self.__title is None:
        raise WQXException("Property 'title' is required.")
      line('Title', self.__title)
      if self.__creationTime is None:
        raise WQXException("Attribute 'creationTime' is required.")
      line('CreationTime', self.__creationTime.astimezone().replace(microsecond=0).isoformat())
      if self.__comment is not None:
        line('Comment', self.__comment)
      if self.__dataService is not None:
        line('DataService', self.__dataService)
      if self.__contactInfo is not None:
        line('ContactInfo', self.__contactInfo)
      for x in self.__notification:
        line('Notification', x)
      if self.__sensitivity is not None:
        line('Sensitivity', self.__sensitivity)
      for key in self.__property:
        with tag('Property'):
          line('name', key)
          line('value', self.__property[key])

    return doc.getvalue()



from ..exceptions import WQXException
from .SimpleContent import (
  Time,
  TimeZoneCode
)
from yattag import Doc

class WQXTime:
  """Custom WQX datatype that defines a local time and corresponding time zone in which the time is measured."""

  __time: Time
  __timeZoneCode: TimeZoneCode

  def __init__(self, o=None, *,
    time:Time = None,
    timeZoneCode:TimeZoneCode = None
  ):
    if isinstance(o, WQXTime):
      # Assign attributes from object without typechecking
      self.__time = o.time
      self.__timeZoneCode = o.timeZoneCode
    elif isinstance(o, dict):
      # Assign attributes from dictionary with typechecking
      self.time = o.get('time', default = None)
      self.timeZoneCode = o.get('timeZoneCode', default = None)
    else:
      # Assign attributes from named keywords with typechecking
      self.time = time
      self.timeZoneCode = timeZoneCode

  @property
  def time(self) -> Time:
    return self.__time
  @time.setter
  def time(self, val:Time) -> None:
    self.__time = Time(val)

  @property
  def timeZoneCode(self) -> TimeZoneCode:
    return self.__timeZoneCode
  @timeZoneCode.setter
  def timeZoneCode(self, val:TimeZoneCode) -> None:
    self.__timeZoneCode = TimeZoneCode(val)

  def generateXML(self, name:str = 'WQXTime') -> str:
    doc = Doc()
    line = doc.line
    tag = doc.tag

    with tag(name):
      if self.__time is None:
        raise WQXException("Attribute 'time' is required.")
      line('Time', self.__time)
      if self.__timeZoneCode is None:
        raise WQXException("Attribute 'timeZoneCode' is required.")
      line('TimeZoneCode', self.__timeZoneCode)

    return doc.getvalue()



from ..exceptions import WQXException
from .IndexType import IndexType
from .SimpleContent import (
  CommentText,
  IndexCalculatedDate,
  IndexIdentifier,
  IndexQualifierCode,
  IndexScore,
  MonitoringLocationIdentifier
)
from yattag import Doc

class BiologicalHabitatIndex:
  """This section allows for the reporting of habitat and biotic integrity indices as a representation of water quality conditions."""

  __indexIdentifier: IndexIdentifier
  __indexType: IndexType
  __indexScore: IndexScore
  __indexQualifierCode: IndexQualifierCode
  __indexCommentText: CommentText
  __indexCalculatedDate: IndexCalculatedDate
  __monitoringLocationIdentifier: MonitoringLocationIdentifier

  def __init__(self, o=None, *,
    indexIdentifier:IndexIdentifier = None,
    indexType:IndexType = None,
    indexScore:IndexScore = None,
    indexQualifierCode:IndexQualifierCode = None,
    indexCommentText:CommentText = None,
    indexCalculatedDate:IndexCalculatedDate = None,
    monitoringLocationIdentifier:MonitoringLocationIdentifier = None
  ):
    if isinstance(o, BiologicalHabitatIndex):
      # Assign attributes from object without typechecking
      self.__indexIdentifier = o.indexIdentifier
      self.__indexType = o.indexType
      self.__indexScore = o.indexScore
      self.__indexQualifierCode = o.indexQualifierCode
      self.__indexCommentText = o.indexCommentText
      self.__indexCalculatedDate = o.indexCalculatedDate
      self.__monitoringLocationIdentifier = o.monitoringLocationIdentifier
    elif isinstance(o, dict):
      # Assign attributes from dictionary with typechecking
      self.indexIdentifier = o.get('indexIdentifier', default = None)
      self.indexType = o.get('indexType', default = None)
      self.indexScore = o.get('indexScore', default = None)
      self.indexQualifierCode = o.get('indexQualifierCode', default = None)
      self.indexCommentText = o.get('indexCommentText', default = None)
      self.indexCalculatedDate = o.get('indexCalculatedDate', default = None)
      self.monitoringLocationIdentifier = o.get('monitoringLocationIdentifier', default = None)
    else:
      # Assign attributes from named keywords with typechecking
      self.indexIdentifier = indexIdentifier
      self.indexType = indexType
      self.indexScore = indexScore
      self.indexQualifierCode = indexQualifierCode
      self.indexCommentText = indexCommentText
      self.indexCalculatedDate = indexCalculatedDate
      self.monitoringLocationIdentifier = monitoringLocationIdentifier

  @property
  def indexIdentifier(self) -> IndexIdentifier:
    return self.__indexIdentifier
  @indexIdentifier.setter
  def indexIdentifier(self, val:IndexIdentifier) -> None:
    self.__indexIdentifier = IndexIdentifier(val)

  @property
  def indexType(self) -> IndexType:
    return self.__indexType
  @indexType.setter
  def indexType(self, val:IndexType) -> None:
    self.__indexType = None if val is None else IndexType(val)

  @property
  def indexScore(self) -> IndexScore:
    return self.__indexScore
  @indexScore.setter
  def indexScore(self, val:IndexScore) -> None:
    self.__indexScore = IndexScore(val)

  @property
  def indexQualifierCode(self) -> IndexQualifierCode:
    return self.__indexQualifierCode
  @indexQualifierCode.setter
  def indexQualifierCode(self, val:IndexQualifierCode) -> None:
    self.__indexQualifierCode = None if val is None else IndexQualifierCode(val)

  @property
  def indexCommentText(self) -> CommentText:
    """Free text with general comments concerning the index."""
    return self.__indexCommentText
  @indexCommentText.setter
  def indexCommentText(self, val:CommentText) -> None:
    """Free text with general comments concerning the index."""
    self.__indexCommentText = None if val is None else CommentText(val)

  @property
  def indexCalculatedDate(self) -> IndexCalculatedDate:
    return self.__indexCalculatedDate
  @indexCalculatedDate.setter
  def indexCalculatedDate(self, val:IndexCalculatedDate) -> None:
    self.__indexCalculatedDate = None if val is None else IndexCalculatedDate(val)

  @property
  def monitoringLocationIdentifier(self) -> MonitoringLocationIdentifier:
    return self.__monitoringLocationIdentifier
  @monitoringLocationIdentifier.setter
  def monitoringLocationIdentifier(self, val:MonitoringLocationIdentifier) -> None:
    self.__monitoringLocationIdentifier = MonitoringLocationIdentifier(val)

  def generateXML(self, name:str = 'BiologicalHabitatIndex') -> str:
    doc = Doc()
    asis = doc.asis
    line = doc.line
    tag = doc.tag

    with tag(name):
      if self.__indexIdentifier is None:
        raise WQXException("Attribute 'indexIdentifier' is required.")
      line('IndexIdentifier', self.__indexIdentifier)
      if self.__indexType is None:
        raise WQXException("Attribute 'indexType' is required.")
      asis(self.__indexType.generateXML('IndexType'))
      if self.__indexScore is None:
        raise WQXException("Attribute 'indexScore' is required.")
      line('IndexScore', self.__indexScore)
      if self.__indexQualifierCode is not None:
        line('IndexQualifierCode', self.__indexQualifierCode)
      if self.__indexCommentText is not None:
        line('IndexCommentText', self.__indexCommentText)
      if self.__indexCalculatedDate is not None:
        line('IndexCalculatedDate', self.__indexCalculatedDate)
      if self.__monitoringLocationIdentifier is None:
        raise WQXException("Attribute 'monitoringLocationIdentifier' is required.")
      line('MonitoringLocationIdentifier', self.__monitoringLocationIdentifier)

    return doc.getvalue()



from ..exceptions import WQXException
from .MeasureCompact import MeasureCompact
from .SimpleContent import (
  CountryCode,
  CountyCode,
  HorizontalCollectionMethodName,
  HorizontalCoordinateReferenceSystemDatumName,
  LatitudeMeasure,
  LongitudeMeasure,
  SourceMapScale,
  StateCode,
  VerticalCollectionMethodName,
  VerticalCoordinateReferenceSystemDatumName
)
from yattag import Doc

class MonitoringLocationGeospatial:
  """Monitoring location geographic location."""

  __latitudeMeasure: LatitudeMeasure
  __longitudeMeasure: LongitudeMeasure
  __sourceMapScale: SourceMapScale
  __horizontalAccuracyMeasure: MeasureCompact
  __verticalAccuracyMeasure: MeasureCompact
  __horizontalCollectionMethodName: HorizontalCollectionMethodName
  __horizontalCoordinateReferenceSystemDatumName: HorizontalCoordinateReferenceSystemDatumName
  __verticalMeasure: MeasureCompact
  __verticalCollectionMethodName: VerticalCollectionMethodName
  __verticalCoordinateReferenceSystemDatumName: VerticalCoordinateReferenceSystemDatumName
  __countryCode: CountryCode
  __stateCode: StateCode
  __countyCode: CountyCode

  def __init__(self, o=None, *,
    latitudeMeasure:LatitudeMeasure = None,
    longitudeMeasure:LongitudeMeasure = None,
    sourceMapScale:SourceMapScale = None,
    horizontalAccuracyMeasure:MeasureCompact = None,
    verticalAccuracyMeasure:MeasureCompact = None,
    horizontalCollectionMethodName:HorizontalCollectionMethodName = None,
    horizontalCoordinateReferenceSystemDatumName:HorizontalCoordinateReferenceSystemDatumName = None,
    verticalMeasure:MeasureCompact = None,
    verticalCollectionMethodName:VerticalCollectionMethodName = None,
    verticalCoordinateReferenceSystemDatumName:VerticalCoordinateReferenceSystemDatumName = None,
    countryCode:CountryCode = None,
    stateCode:StateCode = None,
    countyCode:CountyCode = None
  ):
    if isinstance(o, MonitoringLocationGeospatial):
      # Assign attributes from object without typechecking
      self.__latitudeMeasure = o.latitudeMeasure
      self.__longitudeMeasure = o.longitudeMeasure
      self.__sourceMapScale = o.sourceMapScale
      self.__horizontalAccuracyMeasure = o.horizontalAccuracyMeasure
      self.__verticalAccuracyMeasure = o.verticalAccuracyMeasure
      self.__horizontalCollectionMethodName = o.horizontalCollectionMethodName
      self.__horizontalCoordinateReferenceSystemDatumName = o.horizontalCoordinateReferenceSystemDatumName
      self.__verticalMeasure = o.verticalMeasure
      self.__verticalCollectionMethodName = o.verticalCollectionMethodName
      self.__verticalCoordinateReferenceSystemDatumName = o.verticalCoordinateReferenceSystemDatumName
      self.__countryCode = o.countryCode
      self.__stateCode = o.stateCode
      self.__countyCode = o.countyCode
    elif isinstance(o, dict):
      # Assign attributes from dictionary with typechecking
      self.latitudeMeasure = o.get('latitudeMeasure', default = None)
      self.longitudeMeasure = o.get('longitudeMeasure', default = None)
      self.sourceMapScale = o.get('sourceMapScale', default = None)
      self.horizontalAccuracyMeasure = o.get('horizontalAccuracyMeasure', default = None)
      self.verticalAccuracyMeasure = o.get('verticalAccuracyMeasure', default = None)
      self.horizontalCollectionMethodName = o.get('horizontalCollectionMethodName', default = None)
      self.horizontalCoordinateReferenceSystemDatumName = o.get('horizontalCoordinateReferenceSystemDatumName', default = None)
      self.verticalMeasure = o.get('verticalMeasure', default = None)
      self.verticalCollectionMethodName = o.get('verticalCollectionMethodName', default = None)
      self.verticalCoordinateReferenceSystemDatumName = o.get('verticalCoordinateReferenceSystemDatumName', default = None)
      self.countryCode = o.get('countryCode', default = None)
      self.stateCode = o.get('stateCode', default = None)
      self.countyCode = o.get('countyCode', default = None)
    else:
      # Assign attributes from named keywords with typechecking
      self.latitudeMeasure = latitudeMeasure
      self.longitudeMeasure = longitudeMeasure
      self.sourceMapScale = sourceMapScale
      self.horizontalAccuracyMeasure = horizontalAccuracyMeasure
      self.verticalAccuracyMeasure = verticalAccuracyMeasure
      self.horizontalCollectionMethodName = horizontalCollectionMethodName
      self.horizontalCoordinateReferenceSystemDatumName = horizontalCoordinateReferenceSystemDatumName
      self.verticalMeasure = verticalMeasure
      self.verticalCollectionMethodName = verticalCollectionMethodName
      self.verticalCoordinateReferenceSystemDatumName = verticalCoordinateReferenceSystemDatumName
      self.countryCode = countryCode
      self.stateCode = stateCode
      self.countyCode = countyCode

  @property
  def latitudeMeasure(self) -> LatitudeMeasure:
    return self.__latitudeMeasure
  @latitudeMeasure.setter
  def latitudeMeasure(self, val:LatitudeMeasure) -> None:
    self.__latitudeMeasure = None if val is None else LatitudeMeasure(val)

  @property
  def longitudeMeasure(self) -> LongitudeMeasure:
    return self.__longitudeMeasure
  @longitudeMeasure.setter
  def longitudeMeasure(self, val:LongitudeMeasure) -> None:
    self.__longitudeMeasure = None if val is None else LongitudeMeasure(val)

  @property
  def sourceMapScale(self) -> SourceMapScale:
    return self.__sourceMapScale
  @sourceMapScale.setter
  def sourceMapScale(self, val:SourceMapScale) -> None:
    self.__sourceMapScale = None if val is None else SourceMapScale(val)

  @property
  def horizontalAccuracyMeasure(self) -> MeasureCompact:
    """The horizontal measure of the relative accuracy of the latitude and longitude coordinates."""
    return self.__horizontalAccuracyMeasure
  @horizontalAccuracyMeasure.setter
  def horizontalAccuracyMeasure(self, val:MeasureCompact) -> None:
    """The horizontal measure of the relative accuracy of the latitude and longitude coordinates."""
    self.__horizontalAccuracyMeasure = None if val is None else MeasureCompact(val)

  @property
  def verticalAccuracyMeasure(self) -> MeasureCompact:
    """Depth below land surface datum (LSD) to the bottom of the hole on completion of drilling."""
    return self.__verticalAccuracyMeasure
  @verticalAccuracyMeasure.setter
  def verticalAccuracyMeasure(self, val:MeasureCompact) -> None:
    """Depth below land surface datum (LSD) to the bottom of the hole on completion of drilling."""
    self.__verticalAccuracyMeasure = None if val is None else MeasureCompact(val)

  @property
  def horizontalCollectionMethodName(self) -> HorizontalCollectionMethodName:
    return self.__horizontalCollectionMethodName
  @horizontalCollectionMethodName.setter
  def horizontalCollectionMethodName(self, val:HorizontalCollectionMethodName) -> None:
    self.__horizontalCollectionMethodName = None if val is None else HorizontalCollectionMethodName(val)

  @property
  def horizontalCoordinateReferenceSystemDatumName(self) -> HorizontalCoordinateReferenceSystemDatumName:
    return self.__horizontalCoordinateReferenceSystemDatumName
  @horizontalCoordinateReferenceSystemDatumName.setter
  def horizontalCoordinateReferenceSystemDatumName(self, val:HorizontalCoordinateReferenceSystemDatumName) -> None:
    self.__horizontalCoordinateReferenceSystemDatumName = None if val is None else HorizontalCoordinateReferenceSystemDatumName(val)

  @property
  def verticalMeasure(self) -> MeasureCompact:
    """The measure of elevation (i.e., the altitude), above or below a reference datum."""
    return self.__verticalMeasure
  @verticalMeasure.setter
  def verticalMeasure(self, val:MeasureCompact) -> None:
    """The measure of elevation (i.e., the altitude), above or below a reference datum."""
    self.__verticalMeasure = None if val is None else MeasureCompact(val)

  @property
  def verticalCollectionMethodName(self) -> VerticalCollectionMethodName:
    return self.__verticalCollectionMethodName
  @verticalCollectionMethodName.setter
  def verticalCollectionMethodName(self, val:VerticalCollectionMethodName) -> None:
    self.__verticalCollectionMethodName = None if val is None else VerticalCollectionMethodName(val)

  @property
  def verticalCoordinateReferenceSystemDatumName(self) -> VerticalCoordinateReferenceSystemDatumName:
    return self.__verticalCoordinateReferenceSystemDatumName
  @verticalCoordinateReferenceSystemDatumName.setter
  def verticalCoordinateReferenceSystemDatumName(self, val:VerticalCoordinateReferenceSystemDatumName) -> None:
    self.__verticalCoordinateReferenceSystemDatumName = None if val is None else VerticalCoordinateReferenceSystemDatumName(val)

  @property
  def countryCode(self) -> CountryCode:
    return self.__countryCode
  @countryCode.setter
  def countryCode(self, val:CountryCode) -> None:
    self.__countryCode = None if val is None else CountryCode(val)

  @property
  def stateCode(self) -> StateCode:
    return self.__stateCode
  @stateCode.setter
  def stateCode(self, val:StateCode) -> None:
    self.__stateCode = None if val is None else StateCode(val)

  @property
  def countyCode(self) -> CountyCode:
    return self.__countyCode
  @countyCode.setter
  def countyCode(self, val:CountyCode) -> None:
    self.__countyCode = None if val is None else CountyCode(val)

  def generateXML(self, name:str = 'MonitoringLocationGeospatial') -> str:
    doc = Doc()
    asis = doc.asis
    line = doc.line
    tag = doc.tag

    with tag(name):
      if self.__latitudeMeasure is None:
        raise WQXException("Attribute 'LatitudeMeasure' is required.")
      line('LatitudeMeasure', str(self.__latitudeMeasure))
      if self.__longitudeMeasure is None:
        raise WQXException("Attribute 'LongitudeMeasure' is required.")
      line('LongitudeMeasure', str(self.__longitudeMeasure))
      if self.__sourceMapScale is not None:
        line('SourceMapScale', self.__sourceMapScale)
      if self.__horizontalAccuracyMeasure is not None:
        asis(self.__horizontalAccuracyMeasure.generateXML('HorizontalAccuracyMeasure'))
      if self.__verticalAccuracyMeasure is not None:
        asis(self.__verticalAccuracyMeasure.generateXML('VerticalAccuracyMeasure'))
      if self.__horizontalCollectionMethodName is None:
        raise WQXException("Attribute 'HorizontalCollectionMethodName' is required.")
      line('HorizontalCollectionMethodName', self.__horizontalCollectionMethodName)
      if self.__horizontalCoordinateReferenceSystemDatumName is None:
        raise WQXException("Attribute 'HorizontalCoordinateReferenceSystemDatumName' is required.")
      line('HorizontalCoordinateReferenceSystemDatumName', self.__horizontalCoordinateReferenceSystemDatumName)
      if self.__verticalMeasure is not None:
        asis(self.__verticalMeasure.generateXML('VerticalMeasure'))
      if self.__verticalCollectionMethodName is not None:
        line('VerticalCollectionMethodName', self.__verticalCollectionMethodName)
      if self.__verticalCoordinateReferenceSystemDatumName is not None:
        line('VerticalCoordinateReferenceSystemDatumName', self.__verticalCoordinateReferenceSystemDatumName)
      if self.__countryCode is not None:
        line('CountryCode', self.__countryCode)
      if self.__stateCode is not None:
        line('StateCode', self.__stateCode)
      if self.__countyCode is not None:
        line('CountyCode', self.__countyCode)

    return doc.getvalue()



from ..exceptions import WQXException
from .MeasureCompact import MeasureCompact
from .SimpleContent import (
  DetectionQuantitationLimitCommentText,
  DetectionQuantitationLimitTypeName
)
from yattag import Doc

class DetectionQuantitationLimit:
  """Information that describes one of a variety of detection or quantitation limits determined in a laboratory."""

  __detectionQuantitationLimitTypeName: DetectionQuantitationLimitTypeName
  __detectionQuantitationLimitMeasure: MeasureCompact
  __detectionQuantitationLimitCommentText: DetectionQuantitationLimitCommentText

  def __init__(self, o=None, *,
    detectionQuantitationLimitTypeName:DetectionQuantitationLimitTypeName = None,
    detectionQuantitationLimitMeasure:MeasureCompact = None,
    detectionQuantitationLimitCommentText:DetectionQuantitationLimitCommentText = None
  ):
    if isinstance(o, DetectionQuantitationLimit):
      # Assign attributes from object without typechecking
      self.__detectionQuantitationLimitTypeName = o.detectionQuantitationLimitTypeName
      self.__detectionQuantitationLimitMeasure = o.detectionQuantitationLimitMeasure
      self.__detectionQuantitationLimitCommentText = o.detectionQuantitationLimitCommentText
    elif isinstance(o, dict):
      # Assign attributes from dictionary with typechecking
      self.detectionQuantitationLimitTypeName = o.get('detectionQuantitationLimitTypeName', default = None)
      self.detectionQuantitationLimitMeasure = o.get('detectionQuantitationLimitMeasure', default = None)
      self.detectionQuantitationLimitCommentText = o.get('detectionQuantitationLimitCommentText', default = None)
    else:
      # Assign attributes from named keywords with typechecking
      self.detectionQuantitationLimitTypeName = detectionQuantitationLimitTypeName
      self.detectionQuantitationLimitMeasure = detectionQuantitationLimitMeasure
      self.detectionQuantitationLimitCommentText = detectionQuantitationLimitCommentText

  @property
  def detectionQuantitationLimitTypeName(self) -> DetectionQuantitationLimitTypeName:
    return self.__detectionQuantitationLimitTypeName
  @detectionQuantitationLimitTypeName.setter
  def detectionQuantitationLimitTypeName(self, val:DetectionQuantitationLimitTypeName) -> None:
    self.__detectionQuantitationLimitTypeName = DetectionQuantitationLimitTypeName(val)

  @property
  def detectionQuantitationLimitMeasure(self) -> MeasureCompact:
    return self.__detectionQuantitationLimitMeasure
  @detectionQuantitationLimitMeasure.setter
  def detectionQuantitationLimitMeasure(self, val:MeasureCompact) -> None:
    self.__detectionQuantitationLimitMeasure = MeasureCompact(val)

  @property
  def detectionQuantitationLimitCommentText(self) -> DetectionQuantitationLimitCommentText:
    return self.__detectionQuantitationLimitCommentText
  @detectionQuantitationLimitCommentText.setter
  def detectionQuantitationLimitCommentText(self, val:DetectionQuantitationLimitCommentText) -> None:
    self.__detectionQuantitationLimitCommentText = None if val is None else DetectionQuantitationLimitCommentText(val)

  def generateXML(self, name:str = 'DetectionQuantitationLimit') -> str:
    doc = Doc()
    asis = doc.asis
    line = doc.line
    tag = doc.tag

    with tag(name):
      if self.__detectionQuantitationLimitTypeName is None:
        raise WQXException("Attribute 'detectionQuantitationLimitTypeName' is required.")
      line('DetectionQuantitationLimitTypeName', self.__detectionQuantitationLimitTypeName)
      if self.__detectionQuantitationLimitMeasure is None:
        raise WQXException("Attribute 'detectionQuantitationLimitMeasure' is required.")
      asis(self.__detectionQuantitationLimitMeasure.generateXML('DetectionQuantitationLimitMeasure'))
      if self.__detectionQuantitationLimitCommentText is not None:
        line('DetectionQuantitationLimitCommentText', self.__detectionQuantitationLimitCommentText)

    return doc.getvalue()



from ..exceptions import WQXException
from .AttachedBinaryObject import AttachedBinaryObject
from .BiologicalResultDescription import BiologicalResultDescription
from .ComparableAnalyticalMethod import ComparableAnalyticalMethod
from .LabSamplePreparation import LabSamplePreparation
from .ResultAnalyticalMethod import ResultAnalyticalMethod
from .ResultDescription import ResultDescription
from .ResultLabInformation import ResultLabInformation
from typing import List, Union
from yattag import Doc

class Result:
  """Describes the results of a field measurement, observation, or laboratory analysis."""

  __resultDescription: ResultDescription
  __biologicalResultDescription: BiologicalResultDescription
  __attachedBinaryObject: List[AttachedBinaryObject]
  __resultAnalyticalMethod: ResultAnalyticalMethod
  __comparableAnalyticalMethod: ComparableAnalyticalMethod
  __resultLabInformation: ResultLabInformation
  __labSamplePreparation: List[LabSamplePreparation]

  def __init__(self, o=None, *,
    resultDescription:ResultDescription = None,
    biologicalResultDescription:BiologicalResultDescription = None,
    attachedBinaryObject:List[AttachedBinaryObject] = None,
    resultAnalyticalMethod:ResultAnalyticalMethod = None,
    comparableAnalyticalMethod:ComparableAnalyticalMethod = None,
    resultLabInformation:ResultLabInformation = None,
    labSamplePreparation:List[LabSamplePreparation] = None
  ):
    if isinstance(o, Result):
      # Assign attributes from object without typechecking
      self.__resultDescription = o.resultDescription
      self.__biologicalResultDescription = o.biologicalResultDescription
      self.__attachedBinaryObject = o.attachedBinaryObject
      self.__resultAnalyticalMethod = o.resultAnalyticalMethod
      self.__comparableAnalyticalMethod = o.comparableAnalyticalMethod
      self.__resultLabInformation = o.resultLabInformation
      self.__labSamplePreparation = o.labSamplePreparation
    elif isinstance(o, dict):
      # Assign attributes from dictionary with typechecking
      self.resultDescription = o.get('resultDescription', default = None)
      self.biologicalResultDescription = o.get('biologicalResultDescription', default = None)
      self.attachedBinaryObject = o.get('attachedBinaryObject', default = None)
      self.resultAnalyticalMethod = o.get('resultAnalyticalMethod', default = None)
      self.comparableAnalyticalMethod = o.get('comparableAnalyticalMethod', default = None)
      self.resultLabInformation = o.get('resultLabInformation', default = None)
      self.labSamplePreparation = o.get('labSamplePreparation', default = None)
    else:
      # Assign attributes from named keywords with typechecking
      self.resultDescription = resultDescription
      self.biologicalResultDescription = biologicalResultDescription
      self.attachedBinaryObject = attachedBinaryObject
      self.resultAnalyticalMethod = resultAnalyticalMethod
      self.comparableAnalyticalMethod = comparableAnalyticalMethod
      self.resultLabInformation = resultLabInformation
      self.labSamplePreparation = labSamplePreparation

  @property
  def resultDescription(self) -> ResultDescription:
    return self.__resultDescription
  @resultDescription.setter
  def resultDescription(self, val:ResultDescription) -> None:
    self.__resultDescription = None if val is None else ResultDescription(val)

  @property
  def biologicalResultDescription(self) -> BiologicalResultDescription:
    return self.__biologicalResultDescription
  @biologicalResultDescription.setter
  def biologicalResultDescription(self, val:BiologicalResultDescription) -> None:
    self.__biologicalResultDescription = None if val is None else BiologicalResultDescription(val)

  @property
  def attachedBinaryObject(self) -> List[AttachedBinaryObject]:
    return self.__attachedBinaryObject
  @attachedBinaryObject.setter
  def attachedBinaryObject(self, val:Union[AttachedBinaryObject,List[AttachedBinaryObject]]) -> None:
    if val is None:
      self.__attachedBinaryObject = []
    elif isinstance(val, list):
      r:List[AttachedBinaryObject] = []
      for x in val:
        r.append(AttachedBinaryObject(x))
      self.__attachedBinaryObject = r
    else:
      self.__attachedBinaryObject = [AttachedBinaryObject(val)]

  @property
  def resultAnalyticalMethod(self) -> ResultAnalyticalMethod:
    return self.__resultAnalyticalMethod
  @resultAnalyticalMethod.setter
  def resultAnalyticalMethod(self, val:ResultAnalyticalMethod) -> None:
    self.__resultAnalyticalMethod = None if val is None else ResultAnalyticalMethod(val)

  @property
  def comparableAnalyticalMethod(self) -> ComparableAnalyticalMethod:
    return self.__comparableAnalyticalMethod
  @comparableAnalyticalMethod.setter
  def comparableAnalyticalMethod(self, val:ComparableAnalyticalMethod) -> None:
    self.__comparableAnalyticalMethod = None if val is None else ComparableAnalyticalMethod(val)

  @property
  def resultLabInformation(self) -> ResultLabInformation:
    return self.__resultLabInformation
  @resultLabInformation.setter
  def resultLabInformation(self, val:ResultLabInformation) -> None:
    self.__resultLabInformation = None if val is None else ResultLabInformation(val)

  @property
  def labSamplePreparation(self) -> List[LabSamplePreparation]:
    return self.__labSamplePreparation
  @labSamplePreparation.setter
  def labSamplePreparation(self, val:Union[LabSamplePreparation,List[LabSamplePreparation]]) -> None:
    if val is None:
      self.__labSamplePreparation = []
    elif isinstance(val, list):
      r:List[LabSamplePreparation] = []
      for x in val:
        r.append(LabSamplePreparation(x))
      self.__labSamplePreparation = r
    else:
      self.__labSamplePreparation = [LabSamplePreparation(val)]

  def generateXML(self, name:str = 'Result') -> str:
    doc = Doc()
    asis = doc.asis
    tag = doc.tag

    with tag(name):
      if self.__resultDescription is None:
        raise WQXException("Attribute 'resultDescription' is required.")
      asis(self.__resultDescription.generateXML('ResultDescription'))
      if self.__biologicalResultDescription is not None:
        asis(self.__biologicalResultDescription.generateXML('BiologicalResultDescription'))
      for x in self.__attachedBinaryObject:
        asis(x.generateXML('AttachedBinaryObject'))
      if self.__resultAnalyticalMethod is not None:
        asis(self.__resultAnalyticalMethod.generateXML('ResultAnalyticalMethod'))
      if self.__comparableAnalyticalMethod is not None:
        asis(self.__comparableAnalyticalMethod.generateXML('ComparableAnalyticalMethod'))
      if self.__resultLabInformation is not None:
        asis(self.__resultLabInformation.generateXML('ResultLabInformation'))
      for x in self.__labSamplePreparation:
        asis(x.generateXML('LabSamplePreparation'))

    return doc.getvalue()



from .Activity import *
from .ActivityDescription import *
from .ActivityGroup import *
from .ActivityLocation import *
from .ActivityMetric import *
from .ActivityMetricType import *
from .AlternateMonitoringLocationIdentity import *
from .AquiferInformation import *
from .AttachedBinaryObject import *
from .BibliographicReference import *
from .BiologicalActivityDescription import *
from .BiologicalHabitatCollectionInformation import *
from .BiologicalHabitatIndex import *
from .CollectionEffort import *
from .ComparableAnalyticalMethod import *
from .ComparableAnalyticalMethod import *
from .DataQualityIndicator import *
from .DetectionQuantitationLimit import *
from .ElectronicAddress import *
from .Entity_Update_Identifiers import *
from .FrequencyClassInformation import *
from .HorizontalAccuracyMeasure import *
from .IndexType import *
from .LabSamplePreparation import *
from .Measure import *
from .MeasureCompact import *
from .MonitoringLocation import *
from .MonitoringLocationGeospatial import *
from .MonitoringLocationIdentity import MonitoringLocationIdentity
from .NetInformation import *
from .Organization_Delete import *
from .Organization import *
from .OrganizationAddress import *
from .OrganizationDescription import *
from .Project import *
from .ProjectMonitoringLocationWeighting import *
from .ReferenceMethod import *
from .Result import *
from .ResultAnalyticalMethod import *
from .ResultDescription import *
from .ResultLabInformation import *
from .SampleDescription import *
from .SamplePreparation import *
from .SimpleContent import *
from .TaxonomicDetails import *
from .Telephonic import *
from .WellInformation import *
from .WQX_Delete import *
from .WQX_Update_Identifiers import *
from .WQX import *
from .WQXTime import *



from .ReferenceMethod import ReferenceMethod
from .SimpleContent import (
  PreparationEndDate,
  PreparationStartDate,
  SubstanceDilutionFactor
)
from .WQXTime import WQXTime
from yattag import Doc

class LabSamplePreparation:
  """Describes Lab Sample Preparation procedures which may alter the original state of the Sample and produce Lab subsamples.  These Lab Subsamples are analyized and reported by the Lab as Sample results."""

  __labSamplePreparationMethod: ReferenceMethod
  __preparationStartDate: PreparationStartDate
  __preparationStartTime: WQXTime
  __preparationEndDate: PreparationEndDate
  __preparationEndTime: WQXTime
  __substanceDilutionFactor: SubstanceDilutionFactor

  def __init__(self, o=None, *,
    labSamplePreparationMethod:ReferenceMethod = None,
    preparationStartDate:PreparationStartDate = None,
    preparationStartTime:WQXTime = None,
    preparationEndDate:PreparationEndDate = None,
    preparationEndTime:WQXTime = None,
    substanceDilutionFactor:SubstanceDilutionFactor = None
  ):
    if isinstance(o, LabSamplePreparation):
      # Assign attributes from object without typechecking
      self.__labSamplePreparationMethod = o.labSamplePreparationMethod
      self.__preparationStartDate = o.preparationStartDate
      self.__preparationStartTime = o.preparationStartTime
      self.__preparationEndDate = o.preparationEndDate
      self.__preparationEndTime = o.preparationEndTime
      self.__substanceDilutionFactor = o.substanceDilutionFactor
    elif isinstance(o, dict):
      # Assign attributes from dictionary with typechecking
      self.labSamplePreparationMethod = o.get('labSamplePreparationMethod', default = None)
      self.preparationStartDate = o.get('preparationStartDate', default = None)
      self.preparationStartTime = o.get('preparationStartTime', default = None)
      self.preparationEndDate = o.get('preparationEndDate', default = None)
      self.preparationEndTime = o.get('preparationEndTime', default = None)
      self.substanceDilutionFactor = o.get('substanceDilutionFactor', default = None)
    else:
      # Assign attributes from named keywords with typechecking
      self.labSamplePreparationMethod = labSamplePreparationMethod
      self.preparationStartDate = preparationStartDate
      self.preparationStartTime = preparationStartTime
      self.preparationEndDate = preparationEndDate
      self.preparationEndTime = preparationEndTime
      self.substanceDilutionFactor = substanceDilutionFactor

  @property
  def labSamplePreparationMethod(self) -> ReferenceMethod:
    """Identifying information about the method followed to prepare a sample for analysis."""
    return self.__labSamplePreparationMethod
  @labSamplePreparationMethod.setter
  def labSamplePreparationMethod(self, val:ReferenceMethod) -> None:
    """Identifying information about the method followed to prepare a sample for analysis."""
    self.__labSamplePreparationMethod = None if val is None else ReferenceMethod(val)

  @property
  def preparationStartDate(self) -> PreparationStartDate:
    return self.__preparationStartDate
  @preparationStartDate.setter
  def preparationStartDate(self, val:PreparationStartDate) -> None:
    self.__preparationStartDate = None if val is None else PreparationStartDate(val)

  @property
  def preparationStartTime(self) -> WQXTime:
    """The local time when the preparation/extraction of the sample for analysis began."""
    return self.__preparationStartTime
  @preparationStartTime.setter
  def preparationStartTime(self, val:WQXTime) -> None:
    """The local time when the preparation/extraction of the sample for analysis began."""
    self.__preparationStartTime = None if val is None else WQXTime(val)

  @property
  def preparationEndDate(self) -> PreparationEndDate:
    return self.__preparationEndDate
  @preparationEndDate.setter
  def preparationEndDate(self, val:PreparationEndDate) -> None:
    self.__preparationEndDate = None if val is None else PreparationEndDate(val)

  @property
  def preparationEndTime(self) -> WQXTime:
    """The local time when the preparation/extraction of the sample for analysis was finished."""
    return self.__preparationEndTime
  @preparationEndTime.setter
  def preparationEndTime(self, val:WQXTime) -> None:
    """The local time when the preparation/extraction of the sample for analysis was finished."""
    self.__preparationEndTime = None if val is None else WQXTime(val)

  @property
  def substanceDilutionFactor(self) -> SubstanceDilutionFactor:
    return self.__substanceDilutionFactor
  @substanceDilutionFactor.setter
  def substanceDilutionFactor(self, val:SubstanceDilutionFactor) -> None:
    self.__substanceDilutionFactor = None if val is None else SubstanceDilutionFactor(val)

  def generateXML(self, name:str = 'LabSamplePreparation') -> str:
    doc = Doc()
    asis = doc.asis
    line = doc.line
    tag = doc.tag

    with tag(name):
      if self.__labSamplePreparationMethod is not None:
        asis(self.__labSamplePreparationMethod.generateXML('LabSamplePreparationMethod'))
      if self.__preparationStartDate is not None:
        line('PreparationStartDate', self.__preparationStartDate)
      if self.__preparationStartTime is not None:
        asis(self.__preparationStartTime.generateXML('PreparationStartTime'))
      if self.__preparationEndDate is not None:
        line('PreparationEndDate', self.__preparationEndDate)
      if self.__preparationEndTime is not None:
        asis(self.__preparationEndTime.generateXML('PreparationEndTime'))
      if self.__substanceDilutionFactor is not None:
        line('SubstanceDilutionFactor', self.__substanceDilutionFactor)

    return doc.getvalue()



from ..exceptions import WQXException
from .SimpleContent import (
  OrganizationDescriptionText,
  OrganizationFormalName,
  OrganizationIdentifier,
  TribalCode
)
from yattag import Doc

class OrganizationDescription:
  """The particular word(s) regularly connected with a unique framework of authority within which a person or persons act, or are designated to act, towards some purpose."""

  __organizationIdentifier: OrganizationIdentifier
  __organizationFormalName: OrganizationFormalName
  __organizationDescriptionText: OrganizationDescriptionText
  __tribalCode: TribalCode

  def __init__(self, o=None, *,
    organizationIdentifier:OrganizationIdentifier = None,
    organizationFormalName:OrganizationFormalName = None,
    organizationDescriptionText:OrganizationDescriptionText = None,
    tribalCode:TribalCode = None
  ):
    if isinstance(o, OrganizationDescription):
      # Assign attributes from object without typechecking
      self.__organizationIdentifier = o.organizationIdentifier
      self.__organizationFormalName = o.organizationFormalName
      self.__organizationDescriptionText = o.organizationDescriptionText
      self.__tribalCode = o.tribalCode
    elif isinstance(o, dict):
      # Assign attributes from dictionary with typechecking
      self.organizationIdentifier = o.get('organizationIdentifier', default = None)
      self.organizationFormalName = o.get('organizationFormalName', default = None)
      self.organizationDescriptionText = o.get('organizationDescriptionText', default = None)
      self.tribalCode = o.get('tribalCode', default = None)
    else:
      # Assign attributes from named keywords with typechecking
      self.organizationIdentifier = organizationIdentifier
      self.organizationFormalName = organizationFormalName
      self.organizationDescriptionText = organizationDescriptionText
      self.tribalCode = tribalCode

  @property
  def organizationIdentifier(self) -> OrganizationIdentifier:
    return self.__organizationIdentifier
  @organizationIdentifier.setter
  def organizationIdentifier(self, val:OrganizationIdentifier) -> None:
    self.__organizationIdentifier = OrganizationIdentifier(val)

  @property
  def organizationFormalName(self) -> OrganizationFormalName:
    return self.__organizationFormalName
  @organizationFormalName.setter
  def organizationFormalName(self, val:OrganizationFormalName) -> None:
    self.__organizationFormalName = OrganizationFormalName(val)

  @property
  def organizationDescriptionText(self) -> OrganizationDescriptionText:
    return self.__organizationDescriptionText
  @organizationDescriptionText.setter
  def organizationDescriptionText(self, val:OrganizationDescriptionText) -> None:
    self.__organizationDescriptionText = None if val is None else OrganizationDescriptionText(val)

  @property
  def tribalCode(self) -> TribalCode:
    return self.__tribalCode
  @tribalCode.setter
  def tribalCode(self, val:TribalCode) -> None:
    self.__tribalCode = None if val is None else TribalCode(val)

  def generateXML(self, name:str = 'OrganizationDescription') -> str:
    doc = Doc()
    line = doc.line
    tag = doc.tag

    with tag(name):
      if self.organizationIdentifier is None:
        raise WQXException("Attribute 'organizationIdentifier' is required.")
      line('OrganizationIdentifier', self.__organizationIdentifier)
      if self.organizationFormalName is None:
        raise WQXException("Attribute 'organizationFormalName' is required.")
      line('OrganizationFormalName', self.__organizationFormalName)
      if self.__organizationDescriptionText is not None:
        line('OrganizationDescriptionText', self.__organizationDescriptionText)
      if self.__tribalCode is not None:
        line('TribalCode', self.__tribalCode)

    return doc.getvalue()



from ..exceptions import WQXException
from .SimpleContent import (
  ResourceCreatorName,
  ResourceDate,
  ResourceIdentifier,
  ResourcePublisherName,
  ResourceSubjectText,
  ResourceTitleName
)
from yattag import Doc

class BibliographicReference:
  """The descriptors used to identify and catalog an object."""

  __resourceTitleName: ResourceTitleName
  __resourceCreatorName: ResourceCreatorName
  __resourceSubjectText: ResourceSubjectText
  __resourcePublisherName: ResourcePublisherName
  __resourceDate: ResourceDate
  __resourceIdentifier: ResourceIdentifier

  def __init__(self, o=None, *,
    resourceTitleName:ResourceTitleName = None,
    resourceCreatorName:ResourceCreatorName = None,
    resourceSubjectText:ResourceSubjectText = None,
    resourcePublisherName:ResourcePublisherName = None,
    resourceDate:ResourceDate = None,
    resourceIdentifier:ResourceIdentifier = None
  ):
    if isinstance(o, BibliographicReference):
      # Assign attributes from object without typechecking
      self.__resourceTitleName = resourceTitleName
      self.__resourceCreatorName = resourceCreatorName
      self.__resourceSubjectText = resourceSubjectText
      self.__resourcePublisherName = resourcePublisherName
      self.__resourceDate = resourceDate
      self.__resourceIdentifier = resourceIdentifier
    elif isinstance(o, dict):
      # Assign attributes from dictionary with typechecking
      self.resourceTitleName = o.get('resourceTitleName', default = None)
      self.resourceCreatorName = o.get('resourceCreatorName', default = None)
      self.resourceSubjectText = o.get('resourceSubjectText', default = None)
      self.resourcePublisherName = o.get('resourcePublisherName', default = None)
      self.resourceDate = o.get('resourceDate', default = None)
      self.resourceIdentifier = o.get('resourceIdentifier', default = None)
    else:
      # Assign attributes from named keywords with typechecking
      self.resourceTitleName = resourceTitleName
      self.resourceCreatorName = resourceCreatorName
      self.resourceSubjectText = resourceSubjectText
      self.resourcePublisherName = resourcePublisherName
      self.resourceDate = resourceDate
      self.resourceIdentifier = resourceIdentifier

  @property
  def ResourceTitleName(self) -> ResourceTitleName:
    return self.__ResourceTitleName
  @ResourceTitleName.setter
  def ResourceTitleName(self, val:ResourceTitleName) -> None:
    self.__ResourceTitleName = ResourceTitleName(val)

  @property
  def ResourceCreatorName(self) -> ResourceCreatorName:
    return self.__ResourceCreatorName
  @ResourceCreatorName.setter
  def ResourceCreatorName(self, val:ResourceCreatorName) -> None:
    self.__ResourceCreatorName = None if val is None else ResourceCreatorName(val)

  @property
  def ResourceSubjectText(self) -> ResourceSubjectText:
    return self.__ResourceSubjectText
  @ResourceSubjectText.setter
  def ResourceSubjectText(self, val:ResourceSubjectText) -> None:
    self.__ResourceSubjectText = None if val is None else ResourceSubjectText(val)

  @property
  def ResourcePublisherName(self) -> ResourcePublisherName:
    return self.__ResourcePublisherName
  @ResourcePublisherName.setter
  def ResourcePublisherName(self, val:ResourcePublisherName) -> None:
    self.__ResourcePublisherName = None if val is None else ResourcePublisherName(val)

  @property
  def ResourceDate(self) -> ResourceDate:
    return self.__ResourceDate
  @ResourceDate.setter
  def ResourceDate(self, val:ResourceDate) -> None:
    self.__ResourceDate = ResourceDate(val)

  @property
  def ResourceIdentifier(self) -> ResourceIdentifier:
    return self.__ResourceIdentifier
  @ResourceIdentifier.setter
  def ResourceIdentifier(self, val:ResourceIdentifier) -> None:
    self.__ResourceIdentifier = ResourceIdentifier(val)

  def generateXML(self, name:str = 'BibliographicReference') -> str:
    doc = Doc()
    line = doc.line
    tag = doc.tag

    with tag(name):
      if self.__resourceTitleName is None:
        raise WQXException("Attribute 'ResourceTitleName' is required.")
      line('ResourceTitleName', self.__resourceTitleName)
      if self.__resourceCreatorName is not None:
        line('ResourceCreatorName',self.__resourceCreatorName)
      if self.__resourceSubjectText is not None:
        line('ResourceSubjectText',self.__resourceSubjectText)
      if self.__resourcePublisherName is not None:
        line('ResourcePublisherName',self.__resourcePublisherName)
      if self.__resourceDate is None:
        raise WQXException("Attribute 'ResourceDate' is required.")
      line('ResourceDate', self.__resourceDate)
      if self.__resourceIdentifier is None:
        raise WQXException("Attribute 'ResourceIdentifier' is required.")
      line('ResourceIdentifier', self.__resourceIdentifier)

    return doc.getvalue()



#import setuptools
import runpy
from setuptools import setup, find_packages

version_meta = runpy.run_path("./version.py")
VERSION = version_meta["__version__"]

with open("README.md", "r") as fh:
  long_description = fh.read()

def parse_requirements(filename):
        """Load requirements from a pip requirements file."""
        lineiter = (line.strip() for line in open(filename))
        return [line for line in lineiter if line and not line.startswith("#")]

setup(
  name="wqxlib",
  version=VERSION,
  author="Jon Musselwhite",
  author_email="JMusselwhite@wvstateu.edu",
  description="A library to create files for the EPA's WQX service.",
  long_description=long_description,
  long_description_content_type="text/markdown",
  url="https://github.com/FlippingBinary/wqxlib/tree/main/wqxlib-python",
  packages=find_packages(),
  install_requires=parse_requirements("requirements.txt"),
  classifiers=[
    "Development Status :: 4 - Beta",
    "Intended Audience :: Developers",
    "Intended Audience :: Science/Research",
    "Programming Language :: Python :: 3",
    "License :: OSI Approved :: MIT License",
    "Natural Language :: English",
    "Operating System :: OS Independent",
    "Topic :: Scientific/Engineering",
    "Topic :: Scientific/Engineering :: Hydrology",
    "Topic :: Software Development :: Libraries",
    "Topic :: Software Development :: Libraries :: Python Modules",
  ],
  python_requires='>=3.6',
)



from ..exceptions import WQXException
from .MeasureCompact import MeasureCompact
from .SimpleContent import (
  ActivityConductingOrganizationText,
  ActivityEndDate,
  ActivityIdentifier,
  ActivityIdentifierUserSupplied,
  ActivityMediaName,
  ActivityMediaSubdivisionName,
  ActivityRelativeDepthName,
  ActivityStartDate,
  ActivityTypeCode,
  CommentText,
  DepthAltitudeReferencePointText,
  MonitoringLocationIdentifier,
  ProjectIdentifier,
  SamplingComponentName
)
from .WQXTime import WQXTime
from typing import List, Union
from yattag import Doc

class ActivityDescription:
  """Basic identification information for an activity conducted within a project."""

  __activityIdentifier: ActivityIdentifier
  __activityIdentifierUserSupplied: ActivityIdentifierUserSupplied
  __activityTypeCode: ActivityTypeCode
  __activityMediaName: ActivityMediaName
  __activityMediaSubdivisionName: ActivityMediaSubdivisionName
  __activityStartDate: ActivityStartDate
  __activityStartTime: WQXTime
  __activityEndDate: ActivityEndDate
  __activityEndTime: WQXTime
  __activityRelativeDepthName: ActivityRelativeDepthName
  __activityDepthHeightMeasure: MeasureCompact
  __activityTopDepthHeightMeasure: MeasureCompact
  __activityBottomDepthHeightMeasure: MeasureCompact
  __activityDepthAltitudeReferencePointText: DepthAltitudeReferencePointText
  __projectIdentifier: ProjectIdentifier
  __activityConductingOrganizationText: List[ActivityConductingOrganizationText]
  __monitoringLocationIdentifier: MonitoringLocationIdentifier
  __samplingComponentName: SamplingComponentName
  __activityCommentText: CommentText

  def __init__(self, o=None, *,
    activityIdentifier:ActivityIdentifier = None,
    activityIdentifierUserSupplied:ActivityIdentifierUserSupplied = None,
    activityTypeCode:ActivityTypeCode = None,
    activityMediaName:ActivityMediaName = None,
    activityMediaSubdivisionName:ActivityMediaSubdivisionName = None,
    activityStartDate:ActivityStartDate = None,
    activityStartTime:WQXTime = None,
    activityEndDate:ActivityEndDate = None,
    activityEndTime:WQXTime = None,
    activityRelativeDepthName:ActivityRelativeDepthName = None,
    activityDepthHeightMeasure:MeasureCompact = None,
    activityTopDepthHeightMeasure:MeasureCompact = None,
    activityBottomDepthHeightMeasure:MeasureCompact = None,
    activityDepthAltitudeReferencePointText:DepthAltitudeReferencePointText = None,
    projectIdentifier:ProjectIdentifier = None,
    activityConductingOrganizationText:List[ActivityConductingOrganizationText] = None,
    monitoringLocationIdentifier:MonitoringLocationIdentifier = None,
    samplingComponentName:SamplingComponentName = None,
    activityCommentText:CommentText = None
  ):
    if isinstance(o, ActivityDescription):
      # Assign attributes from object without typechecking
      self.__activityIdentifier = o.activityIdentifier
      self.__activityIdentifierUserSupplied = o.activityIdentifierUserSupplied
      self.__activityTypeCode = o.activityTypeCode
      self.__activityMediaName = o.activityMediaName
      self.__activityMediaSubdivisionName = o.activityMediaSubdivisionName
      self.__activityStartDate = o.activityStartDate
      self.__activityStartTime = o.activityStartTime
      self.__activityEndDate = o.activityEndDate
      self.__activityEndTime = o.activityEndTime
      self.__activityRelativeDepthName = o.activityRelativeDepthName
      self.__activityDepthHeightMeasure = o.activityDepthHeightMeasure
      self.__activityTopDepthHeightMeasure = o.activityTopDepthHeightMeasure
      self.__activityBottomDepthHeightMeasure = o.activityBottomDepthHeightMeasure
      self.__activityDepthAltitudeReferencePointText = o.activityDepthAltitudeReferencePointText
      self.__projectIdentifier = o.projectIdentifier
      self.__activityConductingOrganizationText = o.activityConductingOrganizationText
      self.__monitoringLocationIdentifier = o.monitoringLocationIdentifier
      self.__samplingComponentName = o.samplingComponentName
      self.__activityCommentText = o.activityCommentText
    elif isinstance(o, dict):
      # Assign attributes from other ActivityDescription with typechecking
      self.activityIdentifier = o.get('activityIdentifier', default = None)
      self.activityIdentifierUserSupplied = o.get('activityIdentifierUserSupplied', default = None)
      self.activityTypeCode = o.get('activityTypeCode', default = None)
      self.activityMediaName = o.get('activityMediaName', default = None)
      self.activityMediaSubdivisionName = o.get('activityMediaSubdivisionName', default = None)
      self.activityStartDate = o.get('activityStartDate', default = None)
      self.activityStartTime = o.get('activityStartTime', default = None)
      self.activityEndDate = o.get('activityEndDate', default = None)
      self.activityEndTime = o.get('activityEndTime', default = None)
      self.activityRelativeDepthName = o.get('activityRelativeDepthName', default = None)
      self.activityDepthHeightMeasure = o.get('activityDepthHeightMeasure', default = None)
      self.activityTopDepthHeightMeasure = o.get('activityTopDepthHeightMeasure', default = None)
      self.activityBottomDepthHeightMeasure = o.get('activityBottomDepthHeightMeasure', default = None)
      self.activityDepthAltitudeReferencePointText = o.get('activityDepthAltitudeReferencePointText', default = None)
      self.projectIdentifier = o.get('projectIdentifier', default = None)
      self.activityConductingOrganizationText = o.get('activityConductingOrganizationText', default = [])
      self.monitoringLocationIdentifier = o.get('monitoringLocationIdentifier', default = None)
      self.samplingComponentName = o.get('samplingComponentName', default = None)
      self.activityCommentText = o.get('activityCommentText', default = None)
    else:
      # Assign attributes from named keywords with typechecking
      self.activityIdentifier = activityIdentifier
      self.activityIdentifierUserSupplied = activityIdentifierUserSupplied
      self.activityTypeCode = activityTypeCode
      self.activityMediaName = activityMediaName
      self.activityMediaSubdivisionName = activityMediaSubdivisionName
      self.activityStartDate = activityStartDate
      self.activityStartTime = activityStartTime
      self.activityEndDate = activityEndDate
      self.activityEndTime = activityEndTime
      self.activityRelativeDepthName = activityRelativeDepthName
      self.activityDepthHeightMeasure = activityDepthHeightMeasure
      self.activityTopDepthHeightMeasure = activityTopDepthHeightMeasure
      self.activityBottomDepthHeightMeasure = activityBottomDepthHeightMeasure
      self.activityDepthAltitudeReferencePointText = activityDepthAltitudeReferencePointText
      self.projectIdentifier = projectIdentifier
      self.activityConductingOrganizationText = activityConductingOrganizationText
      self.monitoringLocationIdentifier = monitoringLocationIdentifier
      self.samplingComponentName = samplingComponentName
      self.activityCommentText = activityCommentText

  @property
  def activityIdentifier(self) -> ActivityIdentifier:
    return self.__activityIdentifier
  @activityIdentifier.setter
  def activityIdentifier(self, val:ActivityIdentifier) -> None:
    self.__activityIdentifier = None if val is None else ActivityIdentifier(val)

  @property
  def activityIdentifierUserSupplied(self) -> ActivityIdentifierUserSupplied:
    return self.__activityIdentifierUserSupplied
  @activityIdentifierUserSupplied.setter
  def activityIdentifierUserSupplied(self, val:ActivityIdentifierUserSupplied) -> None:
    self.__activityIdentifierUserSupplied = None if val is None else ActivityIdentifierUserSupplied(val)

  @property
  def activityTypeCode(self) -> ActivityTypeCode:
    return self.__activityTypeCode
  @activityTypeCode.setter
  def activityTypeCode(self, val:ActivityTypeCode) -> None:
    self.__activityTypeCode = None if val is None else ActivityTypeCode(val)

  @property
  def activityMediaName(self) -> ActivityMediaName:
    return self.__activityMediaName
  @activityMediaName.setter
  def activityMediaName(self, val:ActivityMediaName) -> None:
    self.__activityMediaName = None if val is None else ActivityMediaName(val)

  @property
  def activityMediaSubdivisionName(self) -> ActivityMediaSubdivisionName:
    return self.__activityMediaSubdivisionName
  @activityMediaSubdivisionName.setter
  def activityMediaSubdivisionName(self, val:ActivityMediaSubdivisionName) -> None:
    self.__activityMediaSubdivisionName = None if val is None else ActivityMediaSubdivisionName(val)

  @property
  def activityStartDate(self) -> ActivityStartDate:
    return self.__activityStartDate
  @activityStartDate.setter
  def activityStartDate(self, val:ActivityStartDate) -> None:
    self.__activityStartDate = None if val is None else ActivityStartDate(year=val.year,month=val.month,day=val.day)

  @property
  def activityStartTime(self) -> WQXTime:
    """The measure of clock time when the field activity began."""
    return self.__activityStartTime
  @activityStartTime.setter
  def activityStartTime(self, val:WQXTime) -> None:
    """The measure of clock time when the field activity began."""
    self.__activityStartTime = None if val is None else WQXTime(val)

  @property
  def activityEndDate(self) -> ActivityEndDate:
    return self.__activityEndDate
  @activityEndDate.setter
  def activityEndDate(self, val:ActivityEndDate) -> None:
    self.__activityEndDate = None if val is None else ActivityEndDate(year=val.year,month=val.month,day=val.day)

  @property
  def activityEndTime(self) -> WQXTime:
    """The measure of clock time when the field activity ended."""
    return self.__activityEndTime
  @activityEndTime.setter
  def activityEndTime(self, val:WQXTime) -> None:
    """The measure of clock time when the field activity ended."""
    self.__activityEndTime = None if val is None else WQXTime(val)

  @property
  def activityRelativeDepthName(self) -> ActivityRelativeDepthName:
    return self.__activityRelativeDepthName
  @activityRelativeDepthName.setter
  def activityRelativeDepthName(self, val:ActivityRelativeDepthName) -> None:
    self.__activityRelativeDepthName = None if val is None else ActivityRelativeDepthName(val)

  @property
  def activityDepthHeightMeasure(self) -> MeasureCompact:
    """A measurement of the vertical location (measured from a reference point) at which an activity occurred."""
    return self.__activityDepthHeightMeasure
  @activityDepthHeightMeasure.setter
  def activityDepthHeightMeasure(self, val:MeasureCompact) -> None:
    """A measurement of the vertical location (measured from a reference point) at which an activity occurred."""
    self.__activityDepthHeightMeasure = None if val is None else MeasureCompact(val)

  @property
  def activityTopDepthHeightMeasure(self) -> MeasureCompact:
    """A measurement of the upper vertical location of a vertical location range (measured from a reference point) at which an activity occurred."""
    return self.__activityTopDepthHeightMeasure
  @activityTopDepthHeightMeasure.setter
  def activityTopDepthHeightMeasure(self, val:MeasureCompact) -> None:
    """A measurement of the upper vertical location of a vertical location range (measured from a reference point) at which an activity occurred."""
    self.__activityTopDepthHeightMeasure = None if val is None else MeasureCompact(val)

  @property
  def activityBottomDepthHeightMeasure(self) -> MeasureCompact:
    """A measurement of the lower vertical location of a vertical location range (measured from a reference point) at which an activity occurred."""
    return self.__activityBottomDepthHeightMeasure
  @activityBottomDepthHeightMeasure.setter
  def activityBottomDepthHeightMeasure(self, val:MeasureCompact) -> None:
    """A measurement of the lower vertical location of a vertical location range (measured from a reference point) at which an activity occurred."""
    self.__activityBottomDepthHeightMeasure = None if val is None else MeasureCompact(val)

  @property
  def activityDepthAltitudeReferencePointText(self) -> DepthAltitudeReferencePointText:
    """The reference used to indicate the datum or reference used to establish the depth/altitude of an activity."""
    return self.__activityDepthAltitudeReferencePointText
  @activityDepthAltitudeReferencePointText.setter
  def activityDepthAltitudeReferencePointText(self, val:DepthAltitudeReferencePointText) -> None:
    """The reference used to indicate the datum or reference used to establish the depth/altitude of an activity."""
    self.__activityDepthAltitudeReferencePointText = None if val is None else DepthAltitudeReferencePointText(val)

  @property
  def projectIdentifier(self) -> ProjectIdentifier:
    return self.__projectIdentifier
  @projectIdentifier.setter
  def projectIdentifier(self, val:ProjectIdentifier) -> None:
    self.__projectIdentifier = None if val is None else ProjectIdentifier(val)

  @property
  def activityConductingOrganizationText(self) -> List[ActivityConductingOrganizationText]:
    return self.__activityConductingOrganizationText
  @activityConductingOrganizationText.setter
  def activityConductingOrganizationText(self, val:Union[ActivityConductingOrganizationText,List[ActivityConductingOrganizationText]]) -> None:
    if val is None:
      self.__activityConductingOrganizationText = []
    elif isinstance(val, list):
      r:List[ActivityConductingOrganizationText] = []
      for x in val:
        r.append(ActivityConductingOrganizationText(x))
      self.__activityConductingOrganizationText = r
    else:
      self.__activityConductingOrganizationText = [ActivityConductingOrganizationText(val)]

  @property
  def monitoringLocationIdentifier(self) -> MonitoringLocationIdentifier:
    return self.__monitoringLocationIdentifier
  @monitoringLocationIdentifier.setter
  def monitoringLocationIdentifier(self, val:MonitoringLocationIdentifier) -> None:
    self.__monitoringLocationIdentifier = None if val is None else MonitoringLocationIdentifier(val)

  @property
  def samplingComponentName(self) -> SamplingComponentName:
    return self.__samplingComponentName
  @samplingComponentName.setter
  def samplingComponentName(self, val:SamplingComponentName) -> None:
    self.__samplingComponentName = None if val is None else SamplingComponentName(val)

  @property
  def activityCommentText(self) -> CommentText:
    """General comments concerning the activity."""
    return self.__activityCommentText
  @activityCommentText.setter
  def activityCommentText(self, val:CommentText) -> None:
    """General comments concerning the activity."""
    self.__activityCommentText = None if val is None else CommentText(val)

  def generateXML(self, name:str = 'ActivityDescription') -> str:
    doc = Doc()
    asis = doc.asis
    line = doc.line
    tag = doc.tag

    with tag(name):
      if self.__activityIdentifier is None:
        raise WQXException("Attribute 'activityIdentifier' is required.")
      line('ActivityIdentifier', self.__activityIdentifier)
      if self.__activityIdentifierUserSupplied is not None:
        line('ActivityIdentifierUserSupplied', self.__activityIdentifierUserSupplied)
      if self.__activityTypeCode is None:
        raise WQXException("Attribute 'activityTypeCode' is required.")
      line('ActivityTypeCode', self.__activityTypeCode)
      if self.__activityMediaName is None:
        raise WQXException("Attribute 'activityMediaName' is required.")
      line('ActivityMediaName', self.__activityMediaName)
      if self.__activityMediaSubdivisionName is not None:
        line('ActivityMediaSubdivisionName', self.__activityMediaSubdivisionName)
      if self.__activityStartDate is None:
        raise WQXException("Attribute 'activityStartDate' is required.")
      line('ActivityStartDate', str(self.__activityStartDate))
      if self.__activityStartTime is not None:
        asis(self.__activityStartTime.generateXML('ActivityStartTime'))
      if self.__activityEndDate is not None:
        line('ActivityEndDate', str(self.__activityEndDate))
      if self.__activityEndTime is not None:
        asis(self.__activityEndTime.generateXML('ActivityEndTime'))
      if self.__activityRelativeDepthName is not None:
        line('ActivityRelativeDepthName', self.__activityRelativeDepthName)
      if self.__activityDepthHeightMeasure is not None:
        asis(self.__activityDepthHeightMeasure.generateXML('ActivityDepthHeightMeasure'))
      if self.__activityTopDepthHeightMeasure is not None:
        asis(self.__activityTopDepthHeightMeasure.generateXML('ActivityTopDepthHeightMeasure'))
      if self.__activityBottomDepthHeightMeasure is not None:
        asis(self.__activityBottomDepthHeightMeasure.generateXML('ActivityBottomDepthHeightMeasure'))
      if self.__activityDepthAltitudeReferencePointText is not None:
        line('ActivityDepthAltitudeReferencePointText', self.__activityDepthAltitudeReferencePointText)
      if self.__projectIdentifier is None or len(self.__projectIdentifier) < 1:
        raise WQXException("Attribute 'projectIdentifier' must be a list of 1 or more ProjectIdentifier objects.")
      line('ProjectIdentifier', self.__projectIdentifier)
      for x in self.__activityConductingOrganizationText:
        line('ActivityConductingOrganizationText', x)
      if self.__monitoringLocationIdentifier is not None:
        line('MonitoringLocationIdentifier', self.__monitoringLocationIdentifier)
      if self.__samplingComponentName is not None:
        line('SamplingComponentName', self.__samplingComponentName)
      if self.__activityCommentText is not None:
        line('ActivityCommentText', self.__activityCommentText)

    return doc.getvalue()



from ..exceptions import WQXException
from .Entity_Update_Identifiers import UpdateIdentifiers
from typing import List, Union
from yattag import Doc

class WQXUpdateIdentifiers:
  """Main Schema used to update identifiers for major entities (projects, monitoring locations, activity, activity groups, and indexes)."""

  __updateIdentifiers: List[UpdateIdentifiers]

  def __init__(self, o=None, *,
    updateIdentifiers:List[UpdateIdentifiers] = None
  ):
    if isinstance(o, WQXUpdateIdentifiers):
      # Assign attributes from object without typechecking
      self.__updateIdentifiers = o.updateIdentifiers
    elif isinstance(o, dict):
      # Assign attributes from dictionary with typechecking
      self.updateIdentifiers = o.get('updateIdentifiers', default = None)
    else:
      # Assign attributes from named keywords with typechecking
      self.updateIdentifiers = updateIdentifiers

  @property
  def updateIdentifiers(self) -> List[UpdateIdentifiers]:
    return self.__updateIdentifiers
  @updateIdentifiers.setter
  def updateIdentifiers(self, val:Union[UpdateIdentifiers,List[UpdateIdentifiers]]) -> None:
    if val is None:
      self.__updateIdentifiers = []
    elif isinstance(val, list):
      r:List[UpdateIdentifiers] = []
      for x in val:
        r.append(UpdateIdentifiers(x))
      self.__updateIdentifiers = r
    else:
      self.__updateIdentifiers = [UpdateIdentifiers(val)]

  def generateXML(self, name = 'UpdateIdentifiers') -> str:
    doc = Doc()
    asis = doc.asis
    tag = doc.tag

    with tag(
      name,
      ('xmlns','http://www.exchangenetwork.net/schema/wqx/3'),
      ('xmlns:xsi','http://www.w3.org/2001/XMLSchema-instance'),
      ('xsi:schemaLocation','http://www.exchangenetwork.net/schema/wqx/3 http://www.exchangenetwork.net/schema/wqx/3/index.xsd')
    ):
      if len(self.__updateIdentifiers) < 1:
        raise WQXException("Attribute 'updateIdentifiers' must be a list of 1 or more UpdateIdentifiers objects.")
      for x in self.__updateIdentifiers:
        asis(x.generateXML('UpdateIdentifiers'))

    return doc.getvalue()



from ..exceptions import WQXException
from .SimpleContent import (
  MethodDescriptionText,
  MethodIdentifier,
  MethodIdentifierContext,
  MethodName,
  MethodQualifierTypeName
)
from yattag import Doc

class ReferenceMethod:
  """Identifies the procedures, processes, and references required to determine the methods used to obtain a result."""

  __methodIdentifier: MethodIdentifier
  __methodIdentifierContext: MethodIdentifierContext
  __methodName: MethodName
  __methodQualifierTypeName: MethodQualifierTypeName
  __methodDescriptionText: MethodDescriptionText

  def __init__(self, o=None, *,
    methodIdentifier:MethodIdentifier = None,
    methodIdentifierContext:MethodIdentifierContext = None,
    methodName:MethodName = None,
    methodQualifierTypeName:MethodQualifierTypeName = None,
    methodDescriptionText:MethodDescriptionText = None
  ):
    if isinstance(o, ReferenceMethod):
      # Assign attributes from object without typechecking
      self.__methodIdentifier = o.methodIdentifier
      self.__methodIdentifierContext = o.methodIdentifierContext
      self.__methodName = o.methodName
      self.__methodQualifierTypeName = o.methodQualifierTypeName
      self.__methodDescriptionText = o.methodDescriptionText
    elif isinstance(o, dict):
      # Assign attributes from dictionary with typechecking
      self.methodIdentifier = o.get('methodIdentifier', default = None)
      self.methodIdentifierContext = o.get('methodIdentifierContext', default = None)
      self.methodName = o.get('methodName', default = None)
      self.methodQualifierTypeName = o.get('methodQualifierTypeName', default = None)
      self.methodDescriptionText = o.get('methodDescriptionText', default = None)
    else:
      # Assign attributes from named keywords with typechecking
      self.methodIdentifier = methodIdentifier
      self.methodIdentifierContext = methodIdentifierContext
      self.methodName = methodName
      self.methodQualifierTypeName = methodQualifierTypeName
      self.methodDescriptionText = methodDescriptionText

  @property
  def methodIdentifier(self) -> MethodIdentifier:
    return self.__methodIdentifier
  @methodIdentifier.setter
  def methodIdentifier(self, val:MethodIdentifier) -> None:
    self.__methodIdentifier = MethodIdentifier(val)

  @property
  def methodIdentifierContext(self) -> MethodIdentifierContext:
    return self.__methodIdentifierContext
  @methodIdentifierContext.setter
  def methodIdentifierContext(self, val:MethodIdentifierContext) -> None:
    self.__methodIdentifierContext = MethodIdentifierContext(val)

  @property
  def methodName(self) -> MethodName:
    return self.__methodName
  @methodName.setter
  def methodName(self, val:MethodName) -> None:
    self.__methodName = MethodName(val)

  @property
  def methodQualifierTypeName(self) -> MethodQualifierTypeName:
    return self.__methodQualifierTypeName
  @methodQualifierTypeName.setter
  def methodQualifierTypeName(self, val:MethodQualifierTypeName) -> None:
    self.__methodQualifierTypeName = None if val is None else MethodQualifierTypeName(val)

  @property
  def methodDescriptionText(self) -> MethodDescriptionText:
    return self.__methodDescriptionText
  @methodDescriptionText.setter
  def methodDescriptionText(self, val:MethodDescriptionText) -> None:
    self.__methodDescriptionText = None if val is None else MethodDescriptionText(val)

  def generateXML(self, name:str = 'ReferenceMethod') -> str:
    doc = Doc()
    line = doc.line
    tag = doc.tag

    with tag(name):
      if self.__methodIdentifier is None:
        raise WQXException("Attribute 'MethodIdentifier' is required.")
      line('MethodIdentifier', self.__methodIdentifier)
      if self.__methodIdentifierContext is None:
        raise WQXException("Attribute 'MethodIdentifierContext' is required.")
      line('MethodIdentifierContext', self.__methodIdentifierContext)
      if self.__methodName is None:
        raise WQXException("Attribute 'MethodName' is required.")
      line('MethodName', self.__methodName)
      if self.__methodQualifierTypeName is not None:
        line('MethodQualifierTypeName', self.__methodQualifierTypeName)
      if self.__methodDescriptionText is not None:
        line('MethodDescriptionText', self.__methodDescriptionText)

    return doc.getvalue()



from ..exceptions import WQXException
from .BibliographicReference import BibliographicReference
from .SimpleContent import (
  IndexTypeIdentifier,
  IndexTypeIdentifierContext,
  IndexTypeName,
  IndexTypeScaleText
)
from yattag import Doc

class IndexType:
  """This section identifies the index type reported as part of a biological or habitat index."""

  __indexTypeIdentifier: IndexTypeIdentifier
  __indexTypeIdentifierContext: IndexTypeIdentifierContext
  __indexTypeName: IndexTypeName
  __indexTypeCitation: BibliographicReference
  __indexTypeScaleText: IndexTypeScaleText

  def __init__(self, o=None, *,
    indexTypeIdentifier:IndexTypeIdentifier = None,
    indexTypeIdentifierContext:IndexTypeIdentifierContext = None,
    indexTypeName:IndexTypeName = None,
    indexTypeCitation:BibliographicReference = None,
    indexTypeScaleText:IndexTypeScaleText = None
  ):
    if isinstance(o, IndexType):
      # Assign attributes from object without typechecking
      self.__indexTypeIdentifier = o.indexTypeIdentifier
      self.__indexTypeIdentifierContext = o.indexTypeIdentifierContext
      self.__indexTypeName = o.indexTypeName
      self.__indexTypeCitation = o.indexTypeCitation
      self.__indexTypeScaleText = o.indexTypeScaleText
    elif isinstance(o, dict):
      # Assign attributes from dictionary with typechecking
      self.indexTypeIdentifier = o.get('indexTypeIdentifier', default = None)
      self.indexTypeIdentifierContext = o.get('indexTypeIdentifierContext', default = None)
      self.indexTypeName = o.get('indexTypeName', default = None)
      self.indexTypeCitation = o.get('indexTypeCitation', default = None)
      self.indexTypeScaleText = o.get('indexTypeScaleText', default = None)
    else:
      # Assign attributes from named keywords with typechecking
      self.indexTypeIdentifier = indexTypeIdentifier
      self.indexTypeIdentifierContext = indexTypeIdentifierContext
      self.indexTypeName = indexTypeName
      self.indexTypeCitation = indexTypeCitation
      self.indexTypeScaleText = indexTypeScaleText

  @property
  def indexTypeIdentifier(self) -> IndexTypeIdentifier:
    return self.__indexTypeIdentifier
  @indexTypeIdentifier.setter
  def indexTypeIdentifier(self, val:IndexTypeIdentifier) -> None:
    self.__indexTypeIdentifier = IndexTypeIdentifier(val)

  @property
  def indexTypeIdentifierContext(self) -> IndexTypeIdentifierContext:
    return self.__indexTypeIdentifierContext
  @indexTypeIdentifierContext.setter
  def indexTypeIdentifierContext(self, val:IndexTypeIdentifierContext) -> None:
    self.__indexTypeIdentifierContext = IndexTypeIdentifierContext(val)

  @property
  def indexTypeName(self) -> IndexTypeName:
    return self.__indexTypeName
  @indexTypeName.setter
  def indexTypeName(self, val:IndexTypeName) -> None:
    self.__indexTypeName = IndexTypeName(val)

  @property
  def indexTypeCitation(self) -> BibliographicReference:
    """Provides additional description of the source that created or defined the index."""
    return self.__indexTypeCitation
  @indexTypeCitation.setter
  def indexTypeCitation(self, val:BibliographicReference) -> None:
    """Provides additional description of the source that created or defined the index."""
    self.__indexTypeCitation = None if val is None else BibliographicReference(val)

  @property
  def indexTypeScaleText(self) -> IndexTypeScaleText:
    return self.__indexTypeScaleText
  @indexTypeScaleText.setter
  def indexTypeScaleText(self, val:IndexTypeScaleText) -> None:
    self.__indexTypeScaleText = None if val is None else IndexTypeScaleText(val)

  def generateXML(self, name:str = 'IndexType') -> str:
    doc = Doc()
    asis = doc.asis
    line = doc.line
    tag = doc.tag

    with tag(name):
      if self.__indexTypeIdentifier is None:
        raise WQXException("Attribute 'indexTypeIdentifier' is required.")
      line('IndexTypeIdentifier', self.__indexTypeIdentifier)
      if self.__indexTypeIdentifierContext is None:
        raise WQXException("Attribute 'indexTypeIdentifierContext' is required.")
      line('IndexTypeIdentifierContext', self.__indexTypeIdentifierContext)
      if self.__indexTypeName is None:
        raise WQXException("Attribute 'indexTypeName' is required.")
      line('IndexTypeName', self.__indexTypeName)
      if self.__indexTypeCitation is not None:
        asis(self.__indexTypeCitation.generateXML('IndexTypeCitation'))
      if self.__indexTypeScaleText is not None:
        line('IndexTypeScaleText', self.__indexTypeScaleText)

    return doc.getvalue()



from ..exceptions import WQXException
from .SimpleContent import (
  MonitoringLocationIdentifier,
  MonitoringLocationIdentifierContext
)
from yattag import Doc

class AlternateMonitoringLocationIdentity:
  """Alternate identifications of a monitoring location."""

  __monitoringLocationIdentifier: MonitoringLocationIdentifier
  __monitoringLocationIdentifierContext: MonitoringLocationIdentifierContext

  def __init__(self, o=None, *,
    monitoringLocationIdentifier:MonitoringLocationIdentifier = None,
    monitoringLocationIdentifierContext:MonitoringLocationIdentifierContext = None
  ):
    if isinstance(o, AlternateMonitoringLocationIdentity):
      # Assign attributes from object without typechecking
      self.__monitoringLocationIdentifier = o.monitoringLocationIdentifier
      self.__monitoringLocationIdentifierContext = o.monitoringLocationIdentifierContext
    elif isinstance(o, dict):
      # Assign attributes from dictionary with typechecking
      self.monitoringLocationIdentifier = o.get('monitoringLocationIdentifier', default = None)
      self.monitoringLocationIdentifierContext = o.get('monitoringLocationIdentifierContext', default = None)
    else:
      # Assign attributes from named keywords with typechecking
      self.monitoringLocationIdentifier = monitoringLocationIdentifier
      self.monitoringLocationIdentifierContext = monitoringLocationIdentifierContext

  @property
  def monitoringLocationIdentifier(self) -> MonitoringLocationIdentifier:
    return self.__monitoringLocationIdentifier
  @monitoringLocationIdentifier.setter
  def monitoringLocationIdentifier(self, val:MonitoringLocationIdentifier) -> None:
    self.__monitoringLocationIdentifier = MonitoringLocationIdentifier(val)

  @property
  def monitoringLocationIdentifierContext(self) -> MonitoringLocationIdentifierContext:
    return self.__monitoringLocationIdentifierContext
  @monitoringLocationIdentifierContext.setter
  def monitoringLocationIdentifierContext(self, val:MonitoringLocationIdentifierContext) -> None:
    self.__monitoringLocationIdentifierContext = MonitoringLocationIdentifierContext(val)

  def generateXML(self, name:str = 'AlternateMonitoringLocationIdentity') -> str:
    doc = Doc()
    line = doc.line
    tag = doc.tag

    with tag(name):
      if self.__monitoringLocationIdentifier is None:
        raise WQXException("Attribute 'MonitoringLocationIdentifier' is required.")
      line('MonitoringLocationIdentifier', self.__monitoringLocationIdentifier)
      if self.__monitoringLocationIdentifierContext is None:
        raise WQXException("Attribute 'MonitoringLocationIdentifierContext' is required.")
      line('MonitoringLocationIdentifierContext', self.__monitoringLocationIdentifierContext)

    return doc.getvalue()



from ..exceptions import WQXException
from .Organization_Delete import OrganizationDelete
from typing import List, Union
from yattag import Doc

class WQXDelete:
  """Main Schema used to delete a portion of water monitoring results from EPA Office of Water system."""

  __organizationDelete: List[OrganizationDelete]

  def __init__(self, o=None, *,
    organizationDelete:List[OrganizationDelete] = None
  ):
    if isinstance(o, WQXDelete):
      # Assign attributes from object without typechecking
      self.__organizationDelete = o.organizationDelete
    elif isinstance(o, dict):
      # Assign attributes from dictionary with typechecking
      self.organizationDelete = o.get('organizationDelete', default = None)
    else:
      # Assign attributes from named keywords with typechecking
      self.organizationDelete = organizationDelete

  @property
  def organizationDelete(self) -> List[OrganizationDelete]:
    return self.__organizationDelete
  @organizationDelete.setter
  def organizationDelete(self, val: Union[OrganizationDelete,List[OrganizationDelete]]) -> None:
    if val is None:
      self.__organizationDelete = []
    elif isinstance(val, list):
      r:List[OrganizationDelete] = []
      for x in val:
        r.append(OrganizationDelete(x))
      self.__organizationDelete = r
    else:
      self.__organizationDelete = [OrganizationDelete(val)]

  def generateXML(self, name = 'WQXDelete'):
    doc = Doc()
    asis = doc.asis
    tag = doc.tag

    with tag(
      name,
      ('xmlns','http://www.exchangenetwork.net/schema/wqx/3'),
      ('xmlns:xsi','http://www.w3.org/2001/XMLSchema-instance'),
      ('xsi:schemaLocation','http://www.exchangenetwork.net/schema/wqx/3 http://www.exchangenetwork.net/schema/wqx/3/index.xsd')
    ):
      if self.__organizationDelete is None:
        raise WQXException("Attribute 'organizationDelete' is required.")
      for x in self.__organizationDelete:
        asis(x.generateXML('OrganizationDelete'))

    return doc.getvalue()



from datetime import date, time
from decimal import Decimal
from typing import Union

# TODO: Add docstrings from XSD annotations.

class ActivityConductingOrganizationText(str):
  """A name of the Organization conducting an activity."""
  def __init__(self, o=''):
    if len(o) > 120:
      raise ValueError("ActivityConductingOrganizationText must be between 0 and 120 characters.")

class ActivityEndDate(date):
  pass

class ActivityGroupIdentifier(str):
  def __init__(self, o=None):
    if not isinstance(o, str) or len(o) < 1 or len(o) > 55:
      raise ValueError("ActivityGroupIdentifier must be between 1 and 55 characters.")

class ActivityGroupName(str):
  def __init__(self, o=''):
    if len(o) > 120:
      raise ValueError("ActivityGroupName must be between 0 and 120 characters.")

class ActivityGroupTypeCode(str):
  def __init__(self, o=None):
    if not isinstance(o, str) or len(o) < 1 or len(o) > 50:
      raise ValueError("ActivityGroupTypeCode must be between 1 and 50 characters.")

class ActivityIdentifier(str):
  def __init__(self, o=None):
    if not isinstance(o, str) or len(o) < 1 or len(o) > 55:
      raise ValueError("ActivityIdentifier must be between 1 and 55 characters.")

class ActivityIdentifierUserSupplied(str):
  def __init__(self, o=''):
    if len(o) > 55:
      raise ValueError("ActivityIdentifierUserSupplied must be between 0 and 55 characters.")

class ActivityLocationDescriptionText(str):
  def __init__(self, o=''):
    if len(o) > 4000:
      raise ValueError("ActivityLocationDescriptionText must be between 0 and 4000 characters.")

class ActivityMediaName(str):
  def __init__(self, o=None):
    if not isinstance(o, str) or len(o) < 1 or len(o) > 20:
      raise ValueError("ActivityMediaName must be between 1 and 20 characters.")

class ActivityMediaSubdivisionName(str):
  def __init__(self, o=''):
    if len(o) > 60:
      raise ValueError("ActivityMediaSubdivisionName must be between 0 and 60 characters.")

class ActivityRelativeDepthName(str):
  def __init__(self, o=''):
    if len(o) > 30:
      raise ValueError("ActivityRelativeDepthName must be between 0 and 30 characters.")

class ActivityStartDate(date):
  pass

class ActivityTypeCode(str):
  def __init__(self, o=None):
    if not isinstance(o, str) or len(o) < 1 or len(o) > 70:
      raise ValueError("ActivityTypeCode must be between 1 and 70 characters.")

class AddressTypeName(str):
  def __init__(self, o=''):
    if len(o) > 8:
      raise ValueError("AddressTypeName must be between 0 and 8 characters.")

class AddressText(str):
  def __init__(self, o=''):
    if len(o) > 50:
      raise ValueError("AddressText must be between 0 and 50 characters.")

class AnalysisEndDate(date):
  pass

class AnalysisStartDate(date):
  pass

class LocalAquiferCode(str):
  def __init__(self, o=''):
    if len(o) > 120:
      raise ValueError("LocalAquiferCode must be between 0 and 120 characters.")

class LocalAquiferCodeContext(str):
  def __init__(self, o=''):
    if len(o) > 35:
      raise ValueError("LocalAquiferCodeContext must be between 0 and 35 characters.")

class LocalAquiferDescriptionText(str):
  def __init__(self, o=''):
    if len(o) > 512:
      raise ValueError("LocalAquiferDescriptionText must be between 0 and 512 characters.")

class LocalAquiferName(str):
  def __init__(self, o=''):
    if len(o) > 255:
      raise ValueError("LocalAquiferName must be between 0 and 255 characters.")

class AquiferTypeName(str):
  def __init__(self, o=''):
    if len(o) > 255:
      raise ValueError("AquiferTypeName must be between 0 and 255 characters.")

class AssemblageSampledName(str):
  def __init__(self, o=''):
    if len(o) > 50:
      raise ValueError("AssemblageSampledName must be between 0 and 50 characters.")

class BiasValue(str):
  def __init__(self, o=''):
    if len(o) > 60:
      raise ValueError("BiasValue must be between 0 and 60 characters.")

class BiologicalIntentName(str):
  def __init__(self, o=''):
    if len(o) > 35:
      raise ValueError("BiologicalIntentName must be between 0 and 35 characters.")

class BiologicalIndividualIdentifier(str):
  def __init__(self, o=''):
    if len(o) > 60:
      raise ValueError("BiologicalIndividualIdentifier must be between 0 and 60 characters.")

class BinaryObjectFileName(str):
  def __init__(self, o=None):
    if not isinstance(o, str) or len(o) < 1 or len(o) > 255:
      raise ValueError("BinaryObjectFileName must be between 1 and 255 characters.")

class BinaryObjectFileTypeCode(str):
  def __init__(self, o=None):
    if not isinstance(o, str) or len(o) < 1 or len(o) > 6:
      raise ValueError("BinaryObjectFileTypeCode must be between 1 and 6 characters.")

class CellFormName(str):
  def __init__(self, o=''):
    if len(o) > 11:
      raise ValueError("CellFormName must be between 0 and 11 characters.")

class CellShapeName(str):
  def __init__(self, o=''):
    if len(o) > 18:
      raise ValueError("CellShapeName must be between 0 and 18 characters.")

class CharacteristicName(str):
  def __init__(self, o=''):
    if len(o) > 255:
      raise ValueError("CharacteristicName must be between 0 and 255 characters.")

class CharacteristicNameUserSupplied(str):
  def __init__(self, o=''):
    if len(o) > 255:
      raise ValueError("CharacteristicNameUserSupplied must be between 0 and 255 characters.")

class ChemicalPreservativeUsedName(str):
  def __init__(self, o=''):
    if len(o) > 250:
      raise ValueError("ChemicalPreservativeUsedName must be between 0 and 250 characters.")

class CollectionDescriptionText(str):
  def __init__(self, o=''):
    if len(o) > 4000:
      raise ValueError("CollectionDescriptionText must be between 0 and 4000 characters.")

class CommentText(str):
  def __init__(self, o=''):
    if len(o) > 4000:
      raise ValueError("CommentText must be between 0 and 4000 characters.")

class ConfidenceIntervalValue(str):
  def __init__(self, o=''):
    if len(o) > 60:
      raise ValueError("ConfidenceIntervalValue must be between 0 and 60 characters.")

class ConstructionDate(date):
  pass

class CountryCode(str):
  def __init__(self, o=''):
    if len(o) > 2:
      raise ValueError("CountryCode must be between 0 and 2 characters.")

class CountyCode(str):
  def __init__(self, o=''):
    if len(o) > 3:
      raise ValueError("CountyCode must be between 0 and 3 characters.")

class DataLoggerLineName(str):
  def __init__(self, o=''):
    if len(o) > 60:
      raise ValueError("DataLoggerLineName must be between 0 and 60 characters.")

class DepthAltitudeReferencePointText(str):
  def __init__(self, o=''):
    if len(o) > 125:
      raise ValueError("DepthAltitudeReferencePointText must be between 0 and 125 characters.")

class DetectionQuantitationLimitTypeName(str):
  def __init__(self, o=''):
    if len(o) > 35:
      raise ValueError("DetectionQuantitationLimitTypeName must be between 0 and 35 characters.")

class DetectionQuantitationLimitCommentText(str):
  def __init__(self, o=''):
    if len(o) > 4000:
      raise ValueError("DetectionQuantitationLimitCommentText must be between 0 and 4000 characters.")

class ElectronicAddressText(str):
  def __init__(self, o=''):
    if len(o) > 120:
      raise ValueError("ElectronicAddressText must be between 0 and 120 characters.")

class ElectronicAddressTypeName(str):
  def __init__(self, o=''):
    if len(o) > 8:
      raise ValueError("ElectronicAddressTypeName must be between 0 and 8 characters.")

class RecordIdentifierUserSupplied(str):
  def __init__(self, o=''):
    if len(o) > 60:
      raise ValueError("RecordIdentifierUserSupplied must be between 0 and 60 characters.")

class FormulaDescriptionText(str):
  def __init__(self, o=''):
    if len(o) > 4000:
      raise ValueError("FormulaDescriptionText must be between 0 and 4000 characters.")

class FormationTypeText(str):
  def __init__(self, o=''):
    if len(o) > 50:
      raise ValueError("FormationTypeText must be between 0 and 50 characters.")

class FrequencyClassDescriptorCode(str):
  def __init__(self, o=''):
    if len(o) > 50:
      raise ValueError("FrequencyClassDescriptorCode must be between 0 and 50 characters.")

class FrequencyClassDescriptorUnitCode(str):
  def __init__(self, o=''):
    if len(o) > 12:
      raise ValueError("FrequencyClassDescriptorUnitCode must be between 0 and 12 characters.")

class FunctionalFeedingGroupName(str):
  def __init__(self, o=''):
    if len(o) > 30:
      raise ValueError("FunctionalFeedingGroupName must be between 0 and 30 characters.")

class GearProcedureUnitCode(str):
  def __init__(self, o=''):
    if len(o) > 35:
      raise ValueError("GearProcedureUnitCode must be between 0 and 35 characters.")

class GroupSummaryCount(str):
  def __init__(self, o=''):
    if len(o) > 60:
      raise ValueError("GroupSummaryCount must be between 0 and 60 characters.")

class HabitName(str):
  def __init__(self, o=''):
    if len(o) > 15:
      raise ValueError("HabitName must be between 0 and 15 characters.")

class HabitatSelectionMethod(str):
  def __init__(self, o=''):
    if len(o) > 35:
      raise ValueError("HabitatSelectionMethod must be between 0 and 35 characters.")

class HorizontalCollectionMethodName(str):
  def __init__(self, o=None):
    if not isinstance(o, str) or len(o) < 1 or len(o) > 150:
      raise ValueError("HorizontalCollectionMethodName must be between 1 and 150 characters.")

class HorizontalCoordinateReferenceSystemDatumName(str):
  def __init__(self, o=None):
    if not isinstance(o, str) or len(o) < 1 or len(o) > 6:
      raise ValueError("HorizontalCoordinateReferenceSystemDatumName must be between 1 and 6 characters.")

class HUCEightDigitCode(str):
  def __init__(self, o=''):
    if len(o) > 8:
      raise ValueError("HUCEightDigitCode must be between 0 and 8 characters.")

class HUCTwelveDigitCode(str):
  def __init__(self, o=''):
    if len(o) > 12:
      raise ValueError("HUCTwelveDigitCode must be between 0 and 12 characters.")

class HydrologicCondition(str):
  def __init__(self, o=''):
    if len(o) > 60:
      raise ValueError("HydrologicCondition must be between 0 and 60 characters.")

class HydrologicEvent(str):
  def __init__(self, o=''):
    if len(o) > 60:
      raise ValueError("HydrologicEvent must be between 0 and 60 characters.")

class IndexCalculatedDate(date):
  pass

class IndexIdentifier(str):
  def __init__(self, o=None):
    if not isinstance(o, str) or len(o) < 1 or len(o) > 55:
      raise ValueError("IndexIdentifier must be between 1 and 55 characters.")

class IndexQualifierCode(str):
  def __init__(self, o=''):
    if len(o) > 35:
      raise ValueError("IndexQualifierCode must be between 0 and 35 characters.")

class IndexScore(str):
  def __init__(self, o=''):
    if len(o) > 60:
      raise ValueError("IndexScore must be between 0 and 60 characters.")

class IndexTypeIdentifier(str):
  def __init__(self, o=None):
    if not isinstance(o, str) or len(o) < 1 or len(o) > 50:
      raise ValueError("IndexTypeIdentifier must be between 1 and 50 characters.")

class IndexTypeIdentifierContext(str):
  def __init__(self, o=''):
    if len(o) > 50:
      raise ValueError("IndexTypeIdentifierContext must be between 0 and 50 characters.")

class IndexTypeName(str):
  def __init__(self, o=''):
    if len(o) > 100:
      raise ValueError("IndexTypeName must be between 0 and 100 characters.")

class IndexTypeScaleText(str):
  def __init__(self, o=''):
    if len(o) > 50:
      raise ValueError("IndexTypeScaleText must be between 0 and 50 characters.")

class LaboratorySampleSplitRatio(str):
  def __init__(self, o=''):
    if len(o) > 60:
      raise ValueError("LaboratorySampleSplitRatio must be between 0 and 60 characters.")

class LaboratoryAccreditationIndicator(object):
  __o: bool
  def __init__(self, o=False):
    self.__o = bool(o)
  def __str__(self):
    return "True" if self.__o else "False"
  def __bool__(self):
    return self.__o

class LaboratoryAccreditationAuthorityName(str):
  def __init__(self, o=''):
    if len(o) > 20:
      raise ValueError("LaboratoryAccreditationAuthorityName must be between 0 and 20 characters.")

class LaboratoryName(str):
  def __init__(self, o=''):
    if len(o) > 60:
      raise ValueError("LaboratoryName must be between 0 and 60 characters.")

class LatitudeMeasure(Decimal):
  def __new__(self, o):
    s = str(o).split('.')
    totalDigits = 12
    fractionDigits = 10
    s[1] = s[1][0:min(len(s[1]),fractionDigits,max(totalDigits-len(s[0]),0))]
    return super().__new__(self,'.'.join(s))

class LocalityName(str):
  def __init__(self, o=''):
    if len(o) > 30:
      raise ValueError("LocalityName must be between 0 and 30 characters.")

class LocationCategoryName(str):
  def __init__(self, o=''):
    if len(o) > 50:
      raise ValueError("LocationCategoryName must be between 0 and 50 characters.")

class LocationStatusName(str):
  def __init__(self, o=''):
    if len(o) > 15:
      raise ValueError("LocationStatusName must be between 0 and 15 characters.")

class LongitudeMeasure(float):
  def __new__(self,o):
    s = str(o).split('.')
    totalDigits = 14
    fractionDigits = 11
    s[1] = s[1][0:min(len(s[1]),fractionDigits,max(totalDigits-len(s[0]),0))]
    return super().__new__(self,'.'.join(s))

class LowerConfidenceLimitValue(str):
  def __init__(self, o=''):
    if len(o) > 60:
      raise ValueError("LowerConfidenceLimitValue must be between 0 and 60 characters.")

class LowerClassBoundValue(str):
  def __init__(self, o=''):
    if len(o) > 60:
      raise ValueError("LowerClassBoundValue must be between 0 and 60 characters.")

class MeasureQualifierCode(str):
  def __init__(self, o=''):
    if len(o) > 35:
      raise ValueError("MeasureQualifierCode must be between 0 and 35 characters.")

class MeasureUnitCode(str):
  def __init__(self, o=''):
    if len(o) > 12:
      raise ValueError("MeasureUnitCode must be between 0 and 12 characters.")

class MeasureValue(str):
  def __init__(self, o=''):
    if len(o) > 60:
      raise ValueError("MeasureValue must be between 0 and 60 characters.")

class MeasureValueTargeted(str):
  def __init__(self, o=''):
    if len(o) > 60:
      raise ValueError("MeasureValueTargeted must be between 0 and 60 characters.")

class MeasureUnitCodeTargeted(str):
  def __init__(self, o=''):
    if len(o) > 12:
      raise ValueError("MeasureUnitCodeTargeted must be between 0 and 12 characters.")

class MethodDescriptionText(str):
  def __init__(self, o=''):
    if len(o) > 4000:
      raise ValueError("MethodDescriptionText must be between 0 and 4000 characters.")

class MethodIdentifier(str):
  def __init__(self, o=None):
    if not isinstance(o, str) or len(o) < 1 or len(o) > 35:
      raise ValueError("MethodIdentifier must be between 1 and 35 characters.")

class MethodIdentifierContext(str):
  def __init__(self, o=None):
    if not isinstance(o, str) or len(o) < 1 or len(o) > 120:
      raise ValueError("MethodIdentifierContext must be between 1 and 120 characters.")
  
class MethodModificationText(str):
  def __init__(self, o=''):
    if len(o) > 4000:
      raise ValueError("MethodModificationText must be between 0 and 4000 characters.")

class MethodName(str):
  def __init__(self, o=None):
    if not isinstance(o, str) or len(o) < 1 or len(o) > 250:
      raise ValueError("MethodName must be between 1 and 250 characters.")

class MethodQualifierTypeName(str):
  def __init__(self, o=''):
    if len(o) > 25:
      raise ValueError("MethodQualifierTypeName must be between 0 and 25 characters.")

class MethodSpeciationName(str):
  def __init__(self, o=''):
    if len(o) > 20:
      raise ValueError("MethodSpeciationName must be between 0 and 20 characters.")

class MetricTypeIdentifier(str):
  def __init__(self, o=''):
    if len(o) > 50:
      raise ValueError("MetricTypeIdentifier must be between 0 and 50 characters.")

class MetricTypeIdentifierContext(str):
  def __init__(self, o=''):
    if len(o) > 50:
      raise ValueError("MetricTypeIdentifierContext must be between 0 and 50 characters.")

class MetricTypeName(str):
  def __init__(self, o=''):
    if len(o) > 100:
      raise ValueError("MetricTypeName must be between 0 and 100 characters.")

class MetricSamplingPointPlaceInSeries(str):
  def __init__(self, o=''):
    if len(o) > 60:
      raise ValueError("MetricSamplingPointPlaceInSeries must be between 0 and 60 characters.")

class MetricScore(str):
  def __init__(self, o=''):
    if len(o) > 60:
      raise ValueError("MetricScore must be between 0 and 60 characters.")

class MetricTypeScaleText(str):
  def __init__(self, o=''):
    if len(o) > 50:
      raise ValueError("MetricTypeScaleText must be between 0 and 50 characters.")

class MonitoringLocationDescriptionText(str):
  def __init__(self, o=''):
    if len(o) > 4000:
      raise ValueError("MonitoringLocationDescriptionText must be between 0 and 4000 characters.")

class MonitoringLocationIdentifier(str):
  def __init__(self, o=None):
    if not isinstance(o, str) or len(o) < 1 or len(o) > 55:
      raise ValueError("MonitoringLocationIdentifier must be between 1 and 55 characters.")

class MonitoringLocationIdentifierContext(str):
  def __init__(self, o=None):
    if not isinstance(o, str) or len(o) < 1 or len(o) > 120:
      raise ValueError("MonitoringLocationIdentifierContext must be between 1 and 120 characters.")

class MonitoringLocationName(str):
  def __init__(self, o=None):
    if not isinstance(o, str) or len(o) < 1 or len(o) > 255:
      raise ValueError("MonitoringLocationName must be between 1 and 255 characters.")

class MonitoringLocationTypeName(str):
  def __init__(self, o=None):
    if not isinstance(o, str) or len(o) < 1 or len(o) > 45:
      raise ValueError("MonitoringLocationTypeName must be between 1 and 45 characters.")

class NationalAquiferCode(str):
  def __init__(self, o=''):
    if len(o) > 120:
      raise ValueError("NationalAquiferCode must be between 0 and 120 characters.")

class NetTypeName(str):
  def __init__(self, o=None):
    if not isinstance(o, str) or len(o) < 1 or len(o) > 60:
      raise ValueError("NetTypeName must be between 1 and 60 characters.")

class NewIdentifier(str):
  def __init__(self, o=None):
    if not isinstance(o, str) or len(o) < 1 or len(o) > 55:
      raise ValueError("NewIdentifier must be between 1 and 55 characters.")

class OldIdentifier(str):
  def __init__(self, o=None):
    if not isinstance(o, str) or len(o) < 1 or len(o) > 55:
      raise ValueError("OldIdentifier must be between 1 and 55 characters.")

class OrganizationDescriptionText(str):
  def __init__(self, o=''):
    if len(o) > 500:
      raise ValueError("OrganizationDescriptionText must be between 0 and 500 characters.")

class OrganizationFormalName(str):
  def __init__(self, o=None):
    if not isinstance(o, str) or len(o) < 1 or len(o) > 255:
      raise ValueError("OrganizationFormalName must be between 1 and 255 characters.")

class OrganizationIdentifier(str):
  def __init__(self, o=None):
    if not isinstance(o, str) or len(o) < 1 or len(o) > 35:
      print("OrganizationIdentifier:")
      print(o)
      raise ValueError("OrganizationIdentifier must be between 1 and 35 characters.")

class PassCount(str):
  def __init__(self, o=''):
    if len(o) > 60:
      raise ValueError("PassCount must be between 0 and 60 characters.")

class ProportionSampleProcessedNumeric(float):
  pass

class PostalCode(str):
  def __init__(self, o=''):
    if len(o) > 10:
      raise ValueError("PostalCode must be between 0 and 10 characters.")

class PrecisionValue(str):
  def __init__(self, o=''):
    if len(o) > 60:
      raise ValueError("PrecisionValue must be between 0 and 60 characters.")

class PreparationEndDate(date):
  pass

class PreparationStartDate(date):
  pass

class ProjectDescriptionText(str):
  def __init__(self, o=''):
    if len(o) > 4000:
      raise ValueError("ProjectDescriptionText must be between 0 and 4000 characters.")

class ProjectIdentifier(str):
  def __init__(self, o=None):
    if not isinstance(o, str) or len(o) < 1 or len(o) > 55:
      raise ValueError("ProjectIdentifier must be between 1 and 55 characters.")

class ProjectName(str):
  def __init__(self, o=None):
    if not isinstance(o, str) or len(o) < 1 or len(o) > 512:
      raise ValueError("ProjectName must be between 1 and 512 characters.")

class QAPPApprovedIndicator(object):
  __o: bool
  def __init__(self, o=False):
    self.__o = bool(o)
  def __str__(self):
    return "True" if self.__o else "False"
  def __bool__(self):
    return self.__o

class QAPPApprovalAgencyName(str):
  def __init__(self, o=''):
    if len(o) > 50:
      raise ValueError("QAPPApprovalAgencyName must be between 0 and 50 characters.")

class ReferenceLocationEndDate(date):
  pass

class ReferenceLocationStartDate(date):
  pass

class ReferenceLocationTypeCode(str):
  def __init__(self, o=''):
    if len(o) > 20:
      raise ValueError("ReferenceLocationTypeCode must be between 0 and 20 characters.")

class ResourceCreatorName(str):
  def __init__(self, o=''):
    if len(o) > 120:
      raise ValueError("ResourceCreatorName must be between 0 and 120 characters.")

class ResourceDate(date):
  pass

class ResourceIdentifier(str):
  def __init__(self, o=None):
    if not isinstance(o, str) or len(o) < 1 or len(o) > 255:
      raise ValueError("ResourceIdentifier must be between 1 and 255 characters.")

class ResourcePublisherName(str):
  def __init__(self, o=''):
    if len(o) > 60:
      raise ValueError("ResourcePublisherName must be between 0 and 60 characters.")

class ResourceSubjectText(str):
  def __init__(self, o=''):
    if len(o) > 4000:
      raise ValueError("ResourceSubjectText must be between 0 and 4000 characters.")

class ResourceTitleName(str):
  def __init__(self, o=None):
    if not isinstance(o, str) or len(o) < 1 or len(o) > 120:
      raise ValueError("ResourceTitleName must be between 1 and 120 characters.")

class ResultDetectionConditionText(str):
  def __init__(self, o=''):
    if len(o) > 35:
      raise ValueError("ResultDetectionConditionText must be between 0 and 35 characters.")

class LaboratoryCommentText(str):
  def __init__(self, o=''):
    if len(o) > 4000:
      raise ValueError("LaboratoryCommentText must be between 0 and 4000 characters.")

class ResultMeasureValue(str):
  def __init__(self, o=''):
    if len(o) > 60:
      raise ValueError("ResultMeasureValue must be between 0 and 60 characters.")

class ResultParticleSizeBasisText(str):
  def __init__(self, o=''):
    if len(o) > 40:
      raise ValueError("ResultParticleSizeBasisText must be between 0 and 40 characters.")

class ResultSampleFractionText(str):
  def __init__(self, o=''):
    if len(o) > 25:
      raise ValueError("ResultSampleFractionText must be between 0 and 25 characters.")

class ResultSamplingPointName(str):
  def __init__(self, o=''):
    if len(o) > 120:
      raise ValueError("ResultSamplingPointName must be between 0 and 120 characters.")

class ResultSamplingPointCommentText(str):
  def __init__(self, o=''):
    if len(o) > 4000:
      raise ValueError("ResultSamplingPointCommentText must be between 0 and 4000 characters.")

class ResultSamplingPointType(str):
  def __init__(self, o=''):
    if len(o) > 60:
      raise ValueError("ResultSamplingPointType must be between 0 and 60 characters.")

class ResultSamplingPointPlaceInSeries(str):
  def __init__(self, o=''):
    if len(o) > 60:
      raise ValueError("ResultSamplingPointPlaceInSeries must be between 0 and 60 characters.")
  
class ResultStatusIdentifier(str):
  def __init__(self, o=''):
    if len(o) > 12:
      raise ValueError("ResultStatusIdentifier must be between 0 and 12 characters.")

class ResultTemperatureBasisText(str):
  def __init__(self, o=''):
    if len(o) > 12:
      raise ValueError("ResultTemperatureBasisText must be between 0 and 12 characters.")

class ResultTimeBasisText(str):
  def __init__(self, o=''):
    if len(o) > 12:
      raise ValueError("ResultTimeBasisText must be between 0 and 12 characters.")

class ResultValueTypeName(str):
  def __init__(self, o=''):
    if len(o) > 20:
      raise ValueError("ResultValueTypeName must be between 0 and 20 characters.")

class ResultWeightBasisText(str):
  def __init__(self, o=''):
    if len(o) > 60:
      raise ValueError("ResultWeightBasisText must be between 0 and 60 characters.")

class SampleCollectionEquipmentName(str):
  def __init__(self, o=None):
    if not isinstance(o, str) or len(o) < 1 or len(o) > 60:
      raise ValueError("SampleCollectionEquipmentName must be between 1 and 60 characters.")

class SampleCollectionEquipmentCommentText(str):
  def __init__(self, o=''):
    if len(o) > 4000:
      raise ValueError("SampleCollectionEquipmentCommentText must be between 0 and 4000 characters.")

class SampleContainerColorName(str):
  def __init__(self, o=''):
    if len(o) > 60:
      raise ValueError("SampleContainerColorName must be between 0 and 60 characters.")

class SampleContainerLabelName(str):
  def __init__(self, o=''):
    if len(o) > 60:
      raise ValueError("SampleContainerLabelName must be between 0 and 60 characters.")

class SampleContainerTypeName(str):
  def __init__(self, o=''):
    if len(o) > 60:
      raise ValueError("SampleContainerTypeName must be between 0 and 60 characters.")

class SampleTissueAnatomyName(str):
  def __init__(self, o=''):
    if len(o) > 50:
      raise ValueError("SampleTissueAnatomyName must be between 0 and 50 characters.")

class SampleTransportStorageDescription(str):
  def __init__(self, o=''):
    if len(o) > 1999:
      raise ValueError("SampleTransportStorageDescription must be between 0 and 1999 characters.")

class SamplingComponentName(str):
  def __init__(self, o=''):
    if len(o) > 120:
      raise ValueError("SamplingComponentName must be between 0 and 120 characters.")

class SamplingDesignTypeCode(str):
  def __init__(self, o=None):
    if not isinstance(o, str) or len(o) < 1 or len(o) > 20:
      raise ValueError("SamplingDesignTypeCode must be between 1 and 20 characters.")

class SourceMapScale(str):
  def __init__(self, o=''):
    if len(o) > 60:
      raise ValueError("SourceMapScale must be between 0 and 60 characters.")

class StateCode(str):
  def __init__(self, o=''):
    if len(o) > 2:
      raise ValueError("StateCode must be between 0 and 2 characters.")

class StatisticalBaseCode(str):
  def __init__(self, o=''):
    if len(o) > 25:
      raise ValueError("StatisticalBaseCode must be between 0 and 25 characters.")

class StatisticalNValueNumeric(int):
  def __init__(self, o=0):
    if o < 0:
      raise ValueError("StatisticalNValueNumeric must be a positive integer.")

class StatisticalStratumText(str):
  def __init__(self, o=''):
    if len(o) > 15:
      raise ValueError("StatisticalStratumText must be between 0 and 15 characters.")

class SubjectTaxonomicName(str):
  def __init__(self, o=''):
    if len(o) > 255:
      raise ValueError("SubjectTaxonomicName must be between 0 and 255 characters.")

class SubjectTaxonomicNameUserSupplied(str):
  def __init__(self, o=''):
    if len(o) > 255:
      raise ValueError("SubjectTaxonomicNameUserSupplied must be between 0 and 255 characters.")

class SubjectTaxonomicNameUserSuppliedReferenceText(str):
  def __init__(self, o=''):
    if len(o) > 255:
      raise ValueError("SubjectTaxonomicNameUserSuppliedReferenceText must be between 0 and 255 characters.")

class SubstanceDilutionFactor(str):
  def __init__(self, o=''):
    if len(o) > 60:
      raise ValueError("SubstanceDilutionFactor must be between 0 and 60 characters.")

class SupplementalAddressText(str):
  def __init__(self, o=''):
    if len(o) > 120:
      raise ValueError("SupplementalAddressText must be between 0 and 120 characters.")

class TargetCount(str):
  def __init__(self, o=''):
    if len(o) > 35:
      raise ValueError("TargetCount must be between 0 and 35 characters.")

class TaxonomicPollutionTolerance(str):
  def __init__(self, o=''):
    if len(o) > 30:
      raise ValueError("TaxonomicPollutionTolerance must be between 0 and 30 characters.")

class TaxonomistAccreditationIndicator(object):
  __o: bool
  def __init__(self, o=False):
    self.__o = bool(o)
  def __str__(self):
    return "True" if self.__o else "False"
  def __bool__(self):
    return self.__o

class TaxonomistAccreditationAuthorityName(str):
  def __init__(self, o=''):
    if len(o) > 20:
      raise ValueError("TaxonomistAccreditationAuthorityName must be between 0 and 20 characters.")

class TaxonomicPollutionToleranceScaleText(str):
  def __init__(self, o=''):
    if len(o) > 50:
      raise ValueError("TaxonomicPollutionToleranceScaleText must be between 0 and 50 characters.")

class TelephoneExtensionNumberText(str):
  def __init__(self, o=''):
    if len(o) > 6:
      raise ValueError("TelephoneExtensionNumberText must be between 0 and 6 characters.")

class TelephoneNumberText(str):
  def __init__(self, o=''):
    if len(o) > 15:
      raise ValueError("TelephoneNumberText must be between 0 and 15 characters.")

class TelephoneNumberTypeName(str):
  def __init__(self, o=''):
    if len(o) > 6:
      raise ValueError("TelephoneNumberTypeName must be between 0 and 6 characters.")

class ThermalPreservativeUsedName(str):
  def __init__(self, o=''):
    if len(o) > 250:
      raise ValueError("ThermalPreservativeUsedName must be between 0 and 250 characters.")

class Time(time):
  pass

class TimeZoneCode(str):
  def __init__(self, o=None):
    if not isinstance(o, str) or len(o) < 1 or len(o) > 4:
      raise ValueError("TimeZoneCode must be between 1 and 4 characters.")

class ToxicityTestType(str):
  def __init__(self, o=''):
    if len(o) > 30:
      raise ValueError("ToxicityTestType must be between 0 and 30 characters.")

class TribalCode(str):
  def __init__(self, o=''):
    if len(o) > 3:
      raise ValueError("TribalCode must be between 0 and 3 characters.")

class TribalLandIndicator(str):
  def __new__(self, o:Union[str,bool]):
    if isinstance(o, str):
      if o.lower() in ['true', 't', 'yes', 'y', '1']:
        return super().__new__(self, 'true')
      else:
        return super().__new__(self, 'false')
    else:
      return super().__new__(self, 'true' if bool(o) else 'false')
  def __bool__(self):
    if self == 'true':
      return True
    else:
      return False

class TribalLandName(str):
  def __init__(self, o=''):
    if len(o) > 512:
      raise ValueError("TribalLandName must be between 0 and 512 characters.")

class TrophicLevelName(str):
  def __init__(self, o=''):
    if len(o) > 30:
      raise ValueError("TrophicLevelName must be between 0 and 30 characters.")

class UnidentifiedSpeciesIdentifier(str):
  def __init__(self, o=''):
    if len(o) > 255:
      raise ValueError("UnidentifiedSpeciesIdentifier must be between 0 and 255 characters.")

class UpperConfidenceLimitValue(str):
  def __init__(self, o=''):
    if len(o) > 60:
      raise ValueError("UpperConfidenceLimitValue must be between 0 and 60 characters.")

class UpperClassBoundValue(str):
  def __init__(self, o=''):
    if len(o) > 60:
      raise ValueError("UpperClassBoundValue must be between 0 and 60 characters.")

class VerticalCollectionMethodName(str):
  def __init__(self, o=''):
    if len(o) > 50:
      raise ValueError("VerticalCollectionMethodName must be between 0 and 50 characters.")

class VerticalCoordinateReferenceSystemDatumName(str):
  def __init__(self, o=''):
    if len(o) > 10:
      raise ValueError("VerticalCoordinateReferenceSystemDatumName must be between 0 and 10 characters.")

class VoltinismName(str):
  def __init__(self, o=''):
    if len(o) > 25:
      raise ValueError("VoltinismName must be between 0 and 25 characters.")

class WellTypeText(str):
  def __init__(self, o=''):
    if len(o) > 255:
      raise ValueError("WellTypeText must be between 0 and 255 characters.")




from ..exceptions import WQXException
from .SimpleContent import (
  FrequencyClassDescriptorCode,
  FrequencyClassDescriptorUnitCode,
  LowerClassBoundValue,
  UpperClassBoundValue
)
from yattag import Doc

class FrequencyClassInformation:
  """This section allows for the definition of a subgroup of biological communities by life stage, physical attribute, or abnormality to support frequency class studies."""

  __frequencyClassDescriptorCode: FrequencyClassDescriptorCode
  __frequencyClassDescriptorUnitCode: FrequencyClassDescriptorUnitCode
  __lowerClassBoundValue: LowerClassBoundValue
  __upperClassBoundValue: UpperClassBoundValue

  def __init__(self, o=None, *,
    frequencyClassDescriptorCode:FrequencyClassDescriptorCode = None,
    frequencyClassDescriptorUnitCode:FrequencyClassDescriptorUnitCode = None,
    lowerClassBoundValue:LowerClassBoundValue = None,
    upperClassBoundValue:UpperClassBoundValue = None
  ):
    if isinstance(o, FrequencyClassInformation):
      # Assign attributes from object without typechecking
      self.__frequencyClassDescriptorCode = o.frequencyClassDescriptorCode
      self.__frequencyClassDescriptorUnitCode = o.frequencyClassDescriptorUnitCode
      self.__lowerClassBoundValue = o.lowerClassBoundValue
      self.__upperClassBoundValue = o.upperClassBoundValue
    elif isinstance(o, dict):
      # Assign attributes from dictionary with typechecking
      self.frequencyClassDescriptorCode = o.get('frequencyClassDescriptorCode', default = None)
      self.frequencyClassDescriptorUnitCode = o.get('frequencyClassDescriptorUnitCode', default = None)
      self.lowerClassBoundValue = o.get('lowerClassBoundValue', default = None)
      self.upperClassBoundValue = o.get('upperClassBoundValue', default = None)
    else:
      # Assign attributes from named keywords with typechecking
      self.frequencyClassDescriptorCode = frequencyClassDescriptorCode
      self.frequencyClassDescriptorUnitCode = frequencyClassDescriptorUnitCode
      self.lowerClassBoundValue = lowerClassBoundValue
      self.upperClassBoundValue = upperClassBoundValue

  @property
  def frequencyClassDescriptorCode(self) -> FrequencyClassDescriptorCode:
    return self.__frequencyClassDescriptorCode
  @frequencyClassDescriptorCode.setter
  def frequencyClassDescriptorCode(self, val:FrequencyClassDescriptorCode) -> None:
    self.__frequencyClassDescriptorCode = FrequencyClassDescriptorCode(val)

  @property
  def frequencyClassDescriptorUnitCode(self) -> FrequencyClassDescriptorUnitCode:
    return self.__frequencyClassDescriptorUnitCode
  @frequencyClassDescriptorUnitCode.setter
  def frequencyClassDescriptorUnitCode(self, val:FrequencyClassDescriptorUnitCode) -> None:
    self.__frequencyClassDescriptorUnitCode = None if val is None else FrequencyClassDescriptorUnitCode(val)

  @property
  def lowerClassBoundValue(self) -> LowerClassBoundValue:
    return self.__lowerClassBoundValue
  @lowerClassBoundValue.setter
  def lowerClassBoundValue(self, val:LowerClassBoundValue) -> None:
    self.__lowerClassBoundValue = None if val is None else LowerClassBoundValue(val)

  @property
  def upperClassBoundValue(self) -> UpperClassBoundValue:
    return self.__upperClassBoundValue
  @upperClassBoundValue.setter
  def upperClassBoundValue(self, val:UpperClassBoundValue) -> None:
    self.__upperClassBoundValue = None if val is None else UpperClassBoundValue(val)

  def generateXML(self, name:str = 'FrequencyClassInformation') -> str:
    doc = Doc()
    line = doc.line
    tag = doc.tag

    with tag(name):
      if self.__frequencyClassDescriptorCode is None:
        raise WQXException("Attribute 'frequencyClassDescriptorCode' is required.")
      line( 'FrequencyClassDescriptorCode', self.__frequencyClassDescriptorCode)
      if self.__frequencyClassDescriptorUnitCode is not None:
        line('FrequencyClassDescriptorUnitCode', self.__frequencyClassDescriptorUnitCode)
      if self.__lowerClassBoundValue is not None:
        line('LowerClassBoundValue', self.__lowerClassBoundValue)
      if self.__upperClassBoundValue is not None:
        line('UpperClassBoundValue', self.__upperClassBoundValue)

    return doc.getvalue()
